/**
 * @fileoverview
 *
 * This file handles most of the dom events.
 * Some specialised ui elements are seperated into their own file.
 * For example: tooltips.js and deviceSelector.js
 *
 * Will expand on this description later...
 *
 */

(function(){  /* closure to prevent global pollution */
  
  if (!window.widget) { return; }

  var knownWidth = window.innerWidth;
  var knownHeight = window.innerHeight;

  var settings = widget.preferenceForKey('ui-settings') || {
    showInfo: false,
    showPanel: false
  };
  settings.save = function()
  {
    widget.setPreferenceForKey(this, 'ui-settings');
  };
  
  window.addEventListener('resize', function(evt)
  {
    if (document.documentElement.clientWidth != knownWidth ||
        document.documentElement.clientHeight != knownHeight
    )
    {
      window.resizeTo(knownWidth, knownHeight);
      //ui.failWidget('script'); perhaps this is not reason enough to fail the widget
    }    
  }, true);
  
  function prepareCollapsables()
  {
    document.addEventListener('click', function(evt)
    {
      var target = evt.target;
      
      while (target && target.parentNode && target.parentNode.hasClass)
      {
        if (target.parentNode.hasClass('collapsable')
            && target.parentNode.getElementsByTagName('*')[0] == target)
        {
          target.parentNode.toggleClass('collapsed');
          
          ui.resizeWindow();
          return;
        }
        
        target = target.parentNode;
      }
      
    }, false);
  }
  
  /**
   * Calculates how big the window size should be to fit the ui correctly.
   * @param screenWidth {Number} Optional width of screen to override the current screen's width
   * @param screenWidth {Number} Optional height of screen to override the current screen's height
   * @returns {Array} The width and height of the screen
   */

  function getScreenSize(screenWidth, screenHeight)
  {
    var width = Math.max(screenWidth || Control.device.screen.width || 0, 200)+60;
    var height = Math.max(screenHeight || Control.device.screen.height || 0, 200)+60;
    var ele = null;

    ele = g('panel');
    width += settings.showPanel ? ele.offsetWidth : 0;
    height = Math.max(height, ele.offsetHeight + 56);
    // constant of 56 makes up for things like style margins
    // which do not get calculated into offsetHeight

    height += Control.device.state.loaded ? g('info').offsetHeight : 0;
    return [width,height];
  }

  window.ui = {

    init: function()
    {
      tooltips.init();
      this.updateStatus();
      
      if (Control.config.showOutline)
      {
        g('frameWrapper').addClass('showOutline');
        g('config-showOutline').checked = true;
      }
      
      prepareCollapsables();
      
      g('config-showOutline').onchange = function()
      {
        // set config option
        Control.config.showOutline = this.checked;
        // save it
        widget.setPreferenceForKey(this.checked?'1':'0','config-showOutline');
        // apply it
        g('frameWrapper')[this.checked ? 'addClass':'removeClass']('showOutline');
      };
      
      g('config-animations').checked  = Control.config.animations;
      g('config-animations').onchange = function()
      {
        Control.config.animations = this.checked;
        widget.setPreferenceForKey(this.checked?'1':'0','config-animations');
      };
      
      g('tabs-info').onclick = function()
      {
        showTab('showInfo');
      };
      
      g('tabs-devices').onclick = function()
      {
        showTab('showDevices');
      };
      
      g('tabs-plugins').onclick = function()
      {
        showTab('showPlugins');
      };
      
      var tab = '';
      function showTab(newTab)
      {
        g('container').removeClass('showInfo');
        g('container').removeClass('showDevices');
        g('container').removeClass('showPlugins');
        
        if (newTab == tab)
        {
          tab = '';
          settings.showPanel = false;
        }
        else
        {
          tab = newTab;
          settings.showPanel = true;
          g('container').addClass(newTab)
        }
        
        settings.save();
        ui.resizeWindow();
      }
      
      g('screen-buttons-close').onclick = messenger.close;

      g('screen-title-reload').onclick = function()
      {
        Control.loadLastDevice();
      };
      
      // these two do the same thing (the 'done' button is just a placebo)
      g('config-form').onsubmit =
      g('screen-buttons-config').onclick =
        function()
        {
          g('device').toggleClass('config');
          return false;
        };

      g('screen-buttons-help').onclick = function()
      {
        ui.alert('View help file', 'View the docs/widget_emulator.html file in the emulator directory for more information.');
      };

      g('desktop-error-reload').onclick = function()
      {
        Control.loadWidget(Control.lastWidget);
      };
      g('desktop-error-cancel').onclick = function()
      {
        g('desktop').removeClass('error');
      }
      
      g('rotate').onclick = function()
      {
        this.toggleClass('down')
        Control.rotateDevice();
      }

      g('screen-tools-close').onclick = Control.closeWidget.bind(Control);
      g('screen-tools-dock').onclick = ui.dockWidget;
      g('screen-showInfo').onclick = function()
      {
        settings.showInfo = !settings.showInfo;
        settings.save();

        this.toggleClass('down');
        g('info').style.display = this.hasClass('down') ? 'block' : 'none';

        ui.resizeWindow();
      };

      g('info-storage-clear').addEventListener('click', function(evt)
      {
        evt.preventDefault(); // don't act as a link
        emulator.clearPreferences();
        ui.updateStatus();
      }, false);
      
      g('info-storage-post').addEventListener('click', function(evt)
      {
        var str = "";
        var pref = Control.device.widget.preferences;
        evt.preventDefault(); // don't act as a link
        for (var i in pref)
        {
          if (typeof pref[i] == 'string')
          {
            str += i + ' = ' + pref[i] + '\n';
          }
        }
        opera.postError('Widget Preferences:\n'+str);
      }, false);

      g('colors').addEventListener('click', function(evt)
      {
        if (evt.target instanceof HTMLButtonElement)
        {
          var col = window.getComputedStyle(evt.target, '').backgroundColor;
          g('screen').style.backgroundColor = col;
          widget.setPreferenceForKey(col, 'desktopColor');

          // make sure not to have black text on black background
          g('desktop-widgets').style.color = col == '#000000' ? '#eee' : '';
        }
      }, false);
      
      g('dockBlocker').addEventListener('click', function(evt)
      {
        if (Control.device.widget.mode == 'docked')
        {
          emulator.toggleDocked();
        }
      }, false);
      
      var col = widget.preferenceForKey('desktopColor');

      g('screen').style.backgroundColor = col || '#69A7A6';

      if (col == '#000000')
      {
        g('desktop-widgets').style.color = '#eee';
      }
      
      g('notifier-close').onclick = ui.hideNotification;
      g('notifier-text').onclick = ui.activateNotification;
      
      document.addEventListener('click', function(evt)
      {
        /*
          Ensure anchor tags load in new window
        */
        
        var ele = evt.target;
        var href = '';
        
        while (ele)
        {
          // we have to check the tagName instead of HTMLAnchorElement
          // because that object's prototype is different between the
          // parent and the iframe
          if (ele && 'a' == (ele.tagName||'').toLowerCase()
              && !ele.getAttribute('target')
              && (href = ele.getAttribute('href'))
              && href.indexOf('#') // not *starting* with #
              && href.indexOf('javascript:')) // not *starting* with javascript:
          {
            ele.setAttribute('target', '_blank');
          }
          ele = ele.parentNode;
        }
        
      }, false);
      
    },

    createPropertiesList: function(settings)
    {
      var ul = document.createElement('ul');
      ul.setAttribute('class','propertylist');
      var li = null;
      var span = null;

      var arr = [];
      arr.push('Media: "'+settings.media+'"');
      arr.push('Screen: '+settings.screen.join(' x '));
      arr.push('Storage space: ' +  settings.storage.toBytes());

      settings.rotatable && arr.push('Rotatable');

      arr.push('Dock: ' + (!settings.dock ? 'Not enabled' : settings.dock.join(' x ')));

      settings.chrome && arr.push('Chrome: ' + settings.chrome.join(', '));

      settings.useragent && arr.push('User Agent: '+settings.useragent)

      for (var i=0; i<arr.length; i++)
      {
        li = document.createElement('li');
        span = document.createElement('span');
        span.appendChild(document.createTextNode(arr[i]));
        li.appendChild(span);
        ul.appendChild(li);
      }

      return ul;
    },

    createPluginsList: function(settings)
    {
      var ul = document.createElement('ul');
      ul.setAttribute('class','propertylist');
      var li = null;
      var span = null;
      var plugins = settings.plugins;

      if (!plugins.length)
      {
        return document.createTextNode('None enabled');
      }

      for (var i=0; i<plugins.length; i++)
      {
        li = document.createElement('li');
        span = document.createElement('span');
        span.appendChild(document.createTextNode(plugins[i]));
        li.appendChild(span);
        ul.appendChild(li);
      }

      return ul;
    },
    
    /**
     * Creates a document fragment of the checkboxes and radio buttons
     * for selecting plugins.
     *
     * Appends this fragment to #custom-plugins
     */
    
    createPluginsEditingList: function(useCurrent)
    {
      var groups = [];
      
      var frag = document.createDocumentFragment();
      var label = null;
      var ele = null;
      var group = '';
      
      try {
        for (var name in pluginHandler.plugins)
        {
          group = pluginHandler.plugins[name].group || 'none';
          
          if (!groups[group])
          {
            groups[group] = document.createElement('fieldset');
            frag.appendChild(groups[group]);
            
            if (group != 'none')
            {
              try {
                
              }
              catch(e){opera.postError(e)}
              
              groups[group].appendChild(makeItem('', group));
            }
          }
          
          groups[group].appendChild(makeItem(name, group));
        }
        
        g('custom-plugins').textContent = ''; // clear it first
        g('custom-plugins').appendChild(frag);
      }
      catch(e)
      {
        opera.postError(e);
      }
      
      function makeItem(name, group)
      {
        var label = document.createElement('label');
        var ele = document.createElement('input');
        
        ele.setAttribute('data-name', name);
        
        if (group == 'none')
        {
          // no group, always allowed
          ele.type = 'checkbox';
        }
        else
        {
          // in a group only one can be selected
          ele.type = 'radio';
          ele.name = group;
        }
        
        label.appendChild(ele);
        ele = document.createElement('span');
        ele.appendChild(document.createTextNode(name || '(none)'));
        label.appendChild(ele);
        
        return label;
      }
    },
    
    /**
     * Updates the ui for the new device's screen size etc.
     *
     * @param settings {Array} The deb
     *
     */

    loadDevice : function(device, callback)
    {
      g('rotate').style.display = device.rotatable ? 'block' : 'none';
      g('rotate').removeClass('down');
      g('rotate').disabled = false;

      editing.focusDevice(Control.getCurrentDevice());

      g('info-properties').innerHTML = '';
      g('info-properties').appendChild(ui.createPropertiesList(device));

      g('info-plugins').innerHTML = '';
      g('info-plugins').appendChild(ui.createPluginsList(device));
      
      var ANIMATION_TIME = 300;
      var startTime = new Date();
      var startWidth = Control.device.screen.width;
      var startHeight = Control.device.screen.height;
      var width = device.screen[0];
      var height = device.screen[1];
      var interval = 0;
      
      if (Control.config.animations && startWidth &&
          (width != startWidth || height != startHeight) ) // do animation
      {
        applyGlobalDisable(true);
        ui.resizeWindow.apply(ui, getScreenSize(width, height)); // ensure screen is big enough
        interval = setInterval(nextFrame, 1);
        
        g('scroll').style.display = 'none'; // hide scrollbars during animation
      }
      else // no animations
      {
        setTimeout(done, 1); // ensure asynchronouse callback
      }
      
      function nextFrame()
      {
        var progress = (new Date() - startTime) / ANIMATION_TIME;
        
        if (progress > 0 && progress < 1)
        {
          progress = 1-Math.pow(1-progress, 3); // smoother animation
          
          ui.setScreenSize(startWidth*(1-progress) + width*progress,
                  startHeight*(1-progress) + height*progress);
        }
        else // end of animation
        {
          clearInterval(interval);
          applyGlobalDisable(false);
          done();
        }
      }

      function done()
      {
        ui.setScreenSize(width, height);
        
        g('info').style.display = 'block'; // make sure we can read offsetWidth

        g('scroll').style.display = '';

        callback();
      }

    },

    /**
     * Performs UI adjustments to make the screen size a specific size.
     *
     * @param width {Number} Pixels
     * @param height {Number} Pixels
     */

    setScreenSize: function(width, height)
    {
      var MIN_WIDTH = 200;
      var MIN_HEIGHT = 200;
      
      // These minimums represent the smallest size a screen can be
      // before it starts to cause overlaps of UI elements, if it is
      // below these values then the screen gets smaller but the device
      // frame does not. To understand try loading a screen with a size 50x50
      
      g('screen').style.width = Math.max(width, MIN_WIDTH) + 'px';
      g('screen').style.height = Math.max(height, MIN_HEIGHT) + 'px';
      
      g('screen-content').style.borderLeftWidth =   Math.max(0, Math.floor((MIN_WIDTH-width)/2)) + 'px';
      g('screen-content').style.borderRightWidth =  Math.max(0, Math.ceil((MIN_WIDTH-width)/2)) + 'px';
      g('screen-content').style.borderTopWidth =    Math.max(0, Math.floor((MIN_HEIGHT-height)/2)) + 'px';
      g('screen-content').style.borderBottomWidth = Math.max(0, Math.ceil((MIN_HEIGHT-height)/2)) + 'px';
      
      if (Control.device.state.loaded)
      {
        emulator.applyChromeSize();
      }
    },
    
    dragHandler: (function()
    {
      var startTop = 0;
      var startLeft = 0;
      var x0 = 0;
      var y0 = 0;
      var newX = 0;
      var newY = 0;
      
      var frameWindow = null;
      
      function mousedown(evt)
      {
        startX = Control.device.widget.left;
        startY = Control.device.widget.top;
        
        x0 = evt.clientX;
        y0 = evt.clientY;
        
        g('screen-blocker').style.display = 'block';
        
        document.addEventListener('mousemove', mousemove, false);
        document.addEventListener('mouseup', mouseup, false);
        
        evt.preventDefault();
        
      }
      
      var mousemove = (function(evt)
      {
        newX = startX + evt.clientX - x0;
        newY = startY + evt.clientY - y0;
        
        emulator.moveWidgetTo(newX, newY);
        emulator.applyWidgetDimensions();
      
      }).getDelayedCaller(); // causes the handler function to be async
      
      function mouseup()
      {
        document.removeEventListener('mousemove', mousemove, false);
        document.removeEventListener('mouseup', mouseup, false);
      }
      
      function keydown(evt)
      {
        if (evt.keyCode == 18)
        {
          g('screen-blocker').style.display = 'block';
          frameWrapper.addClass('draggable');
          
          document.addEventListener('mousedown', mousedown, false);
  
          document.                addEventListener('keyup', keyup, false);
          emulator.window.document.addEventListener('keyup', keyup, false);
        }
      }
      
      function keyup(evt)
      {
        if (evt.keyCode == 18)
        {
          g('screen-blocker').style.display = 'none';
          frameWrapper.removeClass('draggable');
          
          document.removeEventListener('mousedown', mousedown, false);
          
          document.                removeEventListener('keyup', keyup, false);
          emulator.window.document.removeEventListener('keyup', keyup, false);
          
          evt.preventDefault();
        }
      }
      
      return {
        enable: function(frameWin)
        {
          frameWindow = frameWin; // store
          document.                addEventListener('keydown', keydown, false);
          emulator.window.document.addEventListener('keydown', keydown, false);
        },
        disable: function()
        {
          try {
            document.                removeEventListener('keydown', keydown, false);
            emulator.window.document.removeEventListener('keydown', keydown, false);
          }
          catch(ignore){}
        }
      }
    })(),
    
    // private properties for show/hide/active notification
    _notificationCount: 0,
    _notificationInterval: 0,
    _notificationCallback: null,
    
    _notificationAnimation: function()
    {
      var count = ++ui._notificationCount;
      
      var off = count/20;
      off = off < 1 ? off : off > 7 ? 8-off : 1;
      
      g('notifier').style.marginBottom = Math.round((off-1)*g('notifier').offsetHeight) + 'px';
      
      if (count == 160)
      {
        ui.hideNotification();
      }
    },
    
    hideNotification: function()
    {
      // stop animation
      clearInterval(ui._notificationInterval);
      
      // clear properties
      ui._notificationInterval = 0;
      ui._notificationCallback = null;
      ui._notificationCount = 0;      
      
      // hide ui elements
      g('notifier-text').textContent = '';
      g('notifier-close').disabled = true;
      g('notifier').style.display = '';
    },
    
    showNotification: function(str, callback)
    {
      ui._notificationCallback = callback;
      ui._notificationCount = 0;
      
      g('notifier').style.display = 'block';
      
      g('notifier-text').textContent = str+'';
      g('notifier-close').disabled = false;
      
      if (!ui._notificationInterval)
      {
        ui._notificationInterval = setInterval(ui._notificationAnimation, 30);
        ui._notificationAnimation();
      }
    },
    
    activateNotification: function()
    {
      /* we want to call hide() before
      we do the callback in case an error
      occurs in the callback (We don't want
      the notifier to still be there, but
      then we need to store a reference to
      the callback because hide will
      destroy it */
      var callback = ui._notificationCallback;
      ui.hideNotification();
      if (callback)
      {
        callback();
      }
    },
    
    /**
     * Displays a popup with the given markup
     *
     * @param markup {Array} // markup described using arrays
     */
    
    showPopup: function(markup)
    {
      var winHeight = window.innerHeight;

      markup = createHTML(markup);

      g('dialogue').innerHTML = '';
      g('dialogue').appendChild(markup);
      g('dialogue').style.display = 'block';
      g('screen-blocker').style.display = 'block';

      document.body.addClass('dialogue');

      ui.resizeWindow(500,170);

      applyGlobalDisable(true, g('dialogue'));

      // focus the submit button
      var input = g('dialogue').getElementsByTagName('input');
      for (var i=0; i<input.length; i++)
      {
        if (input[i].type.toUpperCase() == 'TEXT')
        {
          input[i].focus();
          input[i].select();
          return;
        }
      }
      for (var i=0; i<input.length; i++)
      {
        if (input[i].type.toUpperCase() == 'SUBMIT')
        {
          input[i].focus();
          return;
        }
      }
    },

    hidePopup: function()
    {
      g('dialogue').innerHTML = '';
      g('dialogue').style.display = '';
      g('screen-blocker').style.display = '';
      ui.resizeWindow();
      applyGlobalDisable(false);
      ui.resizeWindow();
      document.body.removeClass('dialogue');
      return false;
    },
    
    alert: (function()
    {
      var queue = [];
      
      var markup = [
        'form',
          'onsubmit', done,
          ['div',
            ['h4',
             ''
            ],
            ['p',
             ''
            ],
          ],
          ['fieldset',
           ['input',
            'type','submit',
            'value','Ok'
           ]
          ]
        ];
      
      function done(evt)
      {
        ui.hidePopup();
        
        var callback = queue.pop().callback;
        
        if (callback) callback();
        
        evt.preventDefault();
        
        if (queue.length)
        {
          showNextAlert();
        }
      }
      
      function showNextAlert()
      {
        markup[3][1][1] = queue[queue.length-1].title;
        markup[3][2][1] = queue[queue.length-1].text;
        
        ui.showPopup(markup, true)
      }
      return function(title, text, callback)
      {
        queue.unshift({
          title: title,
          text: text,
          callback: callback
        });
        
        if (queue.length == 1)
        {
          showNextAlert();
        }
      };
    })(),
    
    /**
    * Creates a question for the user
    *
    * @param title {String} The title of the question
    * @param text {String} The body message
    * @param checkbox {String} If set, then a label to apply to a checkbox question
    * @param callback {String} The callback, gets called with 'true' or 'false'
    */
    
    confirm: (function()
    {
      var queue = [];
      
      var markup = [
        'form',
          'onsubmit', ok,
          ['div',
            ['h4',
             ''
            ],
            ['p',
             ''
            ],
          ],
          ['fieldset',
           ['input',
            'type','button',
            'value','Cancel',
            'onclick',cancel
           ],
           ['input',
            'type','submit',
            'value','Ok'
           ]
          ]
        ];
      
      function ok()
      {
        done(true);
        return false;
      }
      function cancel()
      {
        done(false);
      }
      function onKeypress(evt)
      {
        if (evt.keyCode == 27) // escape key
        {
          done(false);
        }
      }
      function done(ok)
      {
        var checkbox = g('promptCheckbox');
        checkbox = checkbox && checkbox.checked;
        
        document.removeEventListener('keypress', onKeypress, false);
        
        ui.hidePopup();
        
        var callback = queue.pop().callback;
        
        if (callback) callback(ok, checkbox);
        
        if (queue.length)
        {
          showNextPrompt();
        }
      }
      
      function showNextPrompt()
      {
        document.addEventListener('keypress', onKeypress, false);
        
        markup[3][1][1] = queue[queue.length-1].title;
        markup[3][2][1] = queue[queue.length-1].text;
        if (queue[queue.length-1].checkbox)
        {
          markup[3][3] = ['label',
            ['input','type','checkbox','id','promptCheckbox'],
            queue[queue.length-1].checkbox
          ];
        }
        else
        {
          markup[3].length = 3;
        }
        ui.showPopup(markup, true)
      }
      
      return function(title, text, checkbox, callback)
      {
        queue.unshift({
          title: title,
          text: text,
          checkbox: checkbox,
          callback: callback
        });
        
        if (queue.length == 1)
        {
          showNextPrompt();
        }
      };
    })(),
    
    failWidget: function(type)
    {
      g('desktop').style.display = 'block';
      g('desktop').addClass('error');
      g('desktop-error').className = type;
      g('desktop-error-config-path').textContent = Control.device.widget.locationPath+'/config.xml';
      var count = 0;
      var interval = setInterval(function()
      {
        if (count++>4) { clearInterval(interval); }
        g('desktop-error')[count%2?'addClass':'removeClass']('flash');
      },200);
    },

    setIcon: function(src)
    {
      var e_icon = g('info-icon');
      var DEFAULT = 'emulator_files/img/widget-icon.png';

      if (!src)
      {
        e_icon.setAttribute('src', DEFAULT);
      }

      src = 'widget/'+src;

      var pic = new Image();
      pic.onerror = function()
      {
        e_icon.setAttribute('src', DEFAULT);
      };
      pic.onload = function()
      {
        e_icon.setAttribute('src', src);
      };
      pic.src = src;
    },

    showPlugin: (function()
    {
      function onclick()
      {
        this.parentNode.toggleClass('collapsed');
        ui.resizeWindow();
      }
      return function(name, frag)
      {
        if (!frag)
        {
          return;
        }
        var html = document.createElement('div');
        html.setAttribute('class', 'plugin');
        var ele = document.createElement('h4');
        ele.appendChild(document.createTextNode(name));
        ele.addEventListener('click', onclick, false);
        html.appendChild(ele);
        ele = document.createElement('div');
        ele.appendChild(frag);
        html.appendChild(ele);
        g('plugin-ui').appendChild(html);
      }
    })(),

    resizeWindow : function(minW,minH)
    {
      var size = getScreenSize();

      var width = size[0];
      var height = size[1];

      if (width < minW) width = minW;
      if (height< minH) height= minH;

      window.resizeTo(knownWidth = width, knownHeight = height);

    },
    /* this should only be called by Control */
    closeWidget : function()
    {
      g('screen-content').style.display = '';
      g('rotate').disabled = false;
      g('desktop').style.display = 'block';
      scrollbars.fix();
      ui.updateStatus();
    },

    reloadWidget : function()
    {
      if (Control.device.widget.locationPath)
      {
        Control.closeWidget();
        Control.loadWidget(Control.device.widget.locationPath);
      }
      else
      {
        if (Control.lastWidget)
        {
          Control.loadWidget(Control.lastWidget);
        }
      }
      return false;
    },

    dockWidget : function()
    {
      if (Control.device.widget.loaded)
      {
        emulator.toggleDocked();
      }
    },

    updateStatus : function()
    {
      if (Control.device.state.loaded && Control.device.widget.opened)
      {
        var storage = 100*Control.device.widget.storage/Control.device.settings.storage;
        storage = storage>0&&storage<1 ? '< 1%' : Math.floor(storage)+'%';

        g('info-mode').textContent = Control.device.widget.mode;
        g('info-size').textContent = Control.device.widget.width + ' x ' + Control.device.widget.height;
        g('info-position').textContent = (Control.device.widget.mode=='docked' ? 'NA' : (Control.device.widget.left + ', ' + Control.device.widget.top));
        g('info-storage').textContent = storage

        g('info-storage-post').style.display = //
        g('info-storage-clear').style.display = Control.device.widget.storage ? '' : 'none';

        if (settings.showInfo)
        {
          g('info').style.display = 'block';
          g('screen-showInfo').addClass('down');
        }
        else
        {
          g('screen-showInfo').removeClass('down');
          g('info').style.display = 'none'
        }
        g('screen-tools').style.display = 'block';
        g('screen-showInfo').style.display = 'block';
      }
      else
      {
        g('info').style.display = //
        g('screen-tools').style.display = //
        g('screen-showInfo').style.display = 'none';
      }
      ui.resizeWindow();
    }
  }; 

})();
