/**
 *
 * @fileoverview
 *
 * <p>This file handles the 'emulation' of the widget. This includes:</p>
 *
 * <ul>
 *   <li>Creation of the iFrame that contains the widget
 *   <li>Reading of config.xml to get name/size etc.
 *   <li>Windowing environment (window.moveTo etc.)
 *   <li>Handling of links (window.openURL(), anchors, forms)
 *   <li>Notifications
 *   <li>etc.
 * </ul>
 * 
 * loadDevice(string: media, int: width, int: height, int:storage)
 * - this function will prepare the device and show the screen
 * - at the end it will call this.loadWidget()
 *
 * loadWidget()
 * - prepares config.xml for reading and waits for the onload event to call this.analyseWidget()
 *
 * analyseWidget()
 * - reads the config.xml and alerts the user if it is invalid
 * - sets the initial coordinates of the widget based on the width/height defined
 * - calls this.runWidget()
 *
 * runWidget()
 * - sets some more properties
 * - creates the iframe for the widget to run in
 *
 * begin()
 * - gets called by the index.html inside the iframe (while it is loading)
 * - creates all the wrappers and environment stuff for the index.html
 * 
 */

/*
  init
  clearPreferences
  this.clearPreferences = function()
  this.loadDevice = function(device, autoload)
  this.loadWidget = function(path)
  this.runWidget = function()
  this.closeWidget = function()
  this.dragHandler = (function()
  this.moveWidgetTo = function(x, y)
  this.detectJSPlugins = function(configXML)
  this.begin = function(frameWin) // creates the wrappers and such
 */



window.emulator = {
  
  window: null, // emulator window object
  
  /**
   * Clears the preferences of the currently opened widget.
   */
  
  clearPreferences: function()
  {
    Control.device.widget.preferences = {};
    Control.device.widget.storage = 0;
    Control.savePreferences();
  },
  
  /**
   * Loads a new device.
   *
   * @param device {Object} The object describing the device
   * @param autoload {Boolean} If true then a widget is auto loaded
   */
  
  loadDevice: function(settings)
  {
    var width = settings.screen[0];
    var height = settings.screen[1];
    
    g('dockBlocker').style.display = 'none';
    g('device').style.display = 'block';
  
    var d = Control.device; // shorthand
  
    d.settings = copyObject(settings);
    
    d.state.loaded = true;
    d.state.chrome = d.settings.chrome || [0,0,0,0];
    d.state.dock = d.settings.dock || [0,0,0,0];
    
    // media object
    d.media = new Media();
    d.media.type = d.settings.media;
    
    d.media.features['device-width'] = width;
    d.media.features['device-height'] = height;
    d.media.features['device-aspect-ratio'] = width/height;
    /* width, height and -o-widget-mode are set after analysing confix.xml */
    
    // directly exposed to the iframe's window:
    d.screen = {
      width: width,
      height: height,
      availWidth: width - d.state.chrome[1] - d.state.chrome[3],
      availHeight: height - d.state.chrome[0] - d.state.chrome[2]
    };
    
    d.plugins = d.settings.plugins;
    
    d.widget.opened = false;
    d.widget.loaded = false;
    d.widget.began = false;
    
    window.scrollbars.fix(true); // true causes full scrollbar reset
    
    ui.updateStatus();
    
    ui.resizeWindow();
  },
  
  /**
   * Starts the loading process of a widget
   * This function just begins the checking of the XML file
   *
   * @param launchPath {String} The path to the launchable widget (a copy of the real widget)
   * @param locationPath {String} The path where the widget originally resides
   */
  
  loadWidget: function(launchPath, locationPath)
  {
    Control.device.widget.launchPath = launchPath;
    Control.device.widget.locationPath = locationPath;
    g('desktop').removeClass('error');
    g('dockBlocker').style.display = 'none';
    g('screen').removeClass('widgetMinimised');
    g('frameWrapper').innerHTML = '';
    fileio.getFileXML(launchPath+'config.xml', emulator.analyseWidget)
  },
  
  /**
   * Checks which JS Plugins are enabled in a widget's
   * configuration file.
   *
   * @param configXML {Document} The XML Document of the config.xml
   * @returns {Object} An object with string properties deliniating the enabled plugins
   */
  
  detectJSPlugins: function(configXML)
  {
    var security = configXML.getElementsByTagName('security')[0];
    var jsplugins = {};
    var ele = null;
    var str1 = '';
    var str2 = '';
    
    // check for "widget/@jsplugins" then "widget/security/content/@jsplugins"
    // one of these must be set for JS Plugins to be enabled
    try
    {
      str1 = configXML.getAttribute('jsplugins').toLowerCase();
    }
    catch(ignore){}
    
    try
    {
      str2 = security.getElementsByTagName('content')[0].
                      getAttribute('jsplugins').toLowerCase();
    }
    catch(ignore){}
    
    if ((str1 == 'yes'       || str2 == 'yes'       ||
         str1 == '1'         || str2 == '1'         ||
         str1 == 'true'      || str2 == 'true'      ||
         str1 == 'jsplugins' || str2 == 'jsplugins') && str2 != 'no')
    {
      // find the individual jsplugins that are allowed
      ele = security.getElementsByTagName('jsplugin');
      
      for (i=0; i<ele.length; i++)
      {
        if (ele[i].parentNode == security) // must be direct child
        {
          jsplugins[ele[i].getAttribute('src')] = true;
        }
      }
    }
    
    return jsplugins;
  },
  
  /**
   * Checks if webserver ability is requested in a config.xml file
   *
   * @param xml {Object} The config.xml file to check
   * @returns {Boolean} True if webeserver is requested, else false
   */
  
  isWebserverRequested: function(xml)
  {
    var feature = xml.getElementsByTagName('feature');
    var param = null;
    
    var i,j;
    
    for (i=0; i<feature.length; i++)
    {
      param = feature[i].getElementsByTagName('param');
      
      for (j=0; j<param.length; j++)
      {
        if (param[j].getAttribute('name') == 'type' &&
          param[j].getAttribute('value') == 'service')
        {
          return true;
        }
      }
    }
    return false;
  },
  
  /**
   * Move a widget to new coordinates
   *
   * @param x {Integer} New distance from left of screen
   * @param y {Integer} New distance from top of screen
   */
  moveWidgetTo: function(x, y)
  {
    if (Control.device.widget.mode == 'docked')
    {
      return; // docked widgets cannot move
    }
    
    x = parseInt(x);
    y = parseInt(y);
    
    if ( typeof x != 'number' || isNaN(x) || !isFinite(x) ) return;
    if ( typeof y != 'number' || isNaN(y) || !isFinite(y) ) return;
    
    x = Math.max(0, Math.min( x, Control.device.screen.availWidth - Control.device.widget.width) );
    y = Math.max(0, Math.min( y, Control.device.screen.availHeight - Control.device.widget.height) );
    
    // store the values
    Control.device.widget.left = x;
    Control.device.widget.top = y;
  },
  
  /**
   * Resize a widget to new size
   * Fixes parameters if they are not correct.
   * Will move the widget if the new size causes it to hang off the screen.
   * 
   * @param x {Integer} New widget width
   * @param y {Integer} New widget height
   */
  resizeWidgetTo: function(x, y)
  {
    if (Control.device.widget.mode == 'docked')
    {
      return; // docked widgets cannot resize
    }
    
    x = parseInt(x);
    y = parseInt(y);
    if ( typeof x != 'number' || isNaN(x) || !isFinite(x) ) return;
    if ( typeof y != 'number' || isNaN(y) || !isFinite(y) ) return;
    
    x = x<1 ? 1 : x>10000 ? 10000 : x;
    y = y<1 ? 1 : y>10000 ? 10000 : y;
    
    var d = Control.device; // shortcut
    // store the values
    d.widget.width = x;
    d.widget.height = y;
  
    if (
      d.widget.left > 0 && x + d.widget.left > d.screen.availWidth
      ||
      d.widget.top > 0 && y + d.widget.top > d.screen.availHeight
    )
    {
      emulator.moveWidgetTo( Math.min(d.widget.left, d.screen.width-x), Math.min(d.widget.top, d.screen.height-y) );
    }
  },
  
  updateAvailSize: function()
  {
    var d = Control.device;
    
    if (d.widget.mode == 'docked')
    {
      // in docked mode avail space is exactly the size of the dock
      d.screen.availWidth = d.state.dock[0];
      d.screen.availHeight = d.state.dock[1];
    }
    else
    {
      // avail space is screen size minus the chrome
      d.screen.availWidth = d.screen.width-d.state.chrome[1]-d.state.chrome[3];
      d.screen.availHeight = d.screen.height-d.state.chrome[0]-d.state.chrome[2];
    }
  },
  
  /**
   * Pushes the size/position info from Control to the
   * actual widget.
   */
  
  applyWidgetDimensions: function()
  {
    var d = Control.device; // alias
    
    /* set actual dom values */
    emulator.iframe.width = d.widget.width;
    emulator.iframe.height = d.widget.height;
    emulator.iframe.style.top = d.widget.top+'px';
    emulator.iframe.style.left = d.widget.left+'px';
    
    /* broadcast values to the widget */
    emulator.window.screenLeft = d.widget.left;
    emulator.window.screenTop = d.widget.top;
    emulator.window.innerWidth = d.widget.width;
    emulator.window.innerHeight = d.widget.height;
    
    d.media.features['-o-widget-mode'] = d.widget.mode;
    d.media.features['width'] = d.widget.width;
    d.media.features['height'] = d.widget.height;
    
    emulator.window.widget.widgetMode = d.widget.mode;
    
    // output to the developer
    ui.updateStatus();
    
    // ensure scrollbars are correct
    scrollbars.fix();
    
    // make sure media queries are matched correctly
    Control.device.media.matchDocument(emulator.window.document);
  },
  
  applyDockDimensions: function()
  {
    var d = Control.device; // alias
    
    d.widget.mode = d.widget.defaultMode; // ensure we can move the widget
    
    emulator.moveWidgetTo(0,0);
    emulator.resizeWidgetTo(d.state.dock[0], d.state.dock[1]);
    opera.postError(d.state.dock);
    d.widget.mode = 'docked';
    // must set mode after move and resize (otherwise those functions die silently)
    
    emulator.updateAvailSize();
    emulator.applyWidgetDimensions();
    
    g('dockBlocker').style.width = d.widget.width+'px';
    g('dockBlocker').style.height = d.widget.height+'px';
    g('dockBlocker').style.top = d.state.chrome[0]+'px';
    g('dockBlocker').style.left = d.state.chrome[3]+'px';
    g('dockBlocker').style.display = 'block';
    
    g('device').addClass('docked');
    
    scrollbars.fix();
    ui.updateStatus();
  },
  
  /**
   * Will dock the widget.
   * Warning, only use this function if the widget is
   * allowed to be docked.
   * If settings.dockX/Y are not set it will behave unexpectedly.
   */
  
  dockWidget: function()
  {
    var d = Control.device; // alias
    
    d.tempData = { // store old values for when we go back
      top: d.widget.top,
      left: d.widget.left,
      width: d.widget.width,
      height: d.widget.height
    };
    
    emulator.applyDockDimensions();
    
    emulator.window.widget.dispatchEvent({
      type: 'widgetmodechange',
      widgetMode: 'docked'
    });
    emulator.window.widget.dispatchEvent({
      type: 'resolution',
      width: d.screen.availWidth,
      height: d.screen.availHeight
    });
  },
  
  undockWidget: function()
  {
    var d = Control.device; // alias
    
    d.widget.mode = d.widget.defaultMode;
    // must set mode before move and resize (otherwise those functions die silently)
    
    emulator.updateAvailSize();
    
    emulator.moveWidgetTo(d.tempData.left, d.tempData.top);
    emulator.resizeWidgetTo(d.tempData.width, d.tempData.height);
    
    emulator.applyWidgetDimensions();
    
    delete d.tempData; // not needed any more
    
    g('dockBlocker').style.display = 'none';
    g('device').removeClass('docked');
    
    emulator.window.widget.dispatchEvent({
      type: 'widgetmodechange',
      widgetMode: d.widget.mode
    });
    emulator.window.widget.dispatchEvent({
      type: 'resolution',
      width: d.screen.availWidth,
      height: d.screen.availHeight
    });
    
    scrollbars.fix();
    ui.updateStatus();
  },
  
  /**
   * Changes widgetMode
   *
   * <p>This will change the mode of the widget
   * (exmaple from 'widget' to 'docked'). It will
   * make the visible changes (by adjusting the iFrame),
   * set the appropriate values (screen.availWidth etc.)
   * then trigger the events on the widget object.</p>
   *
   * @param {string} mode The new mode to change to
   */
  
  changeWidgetMode: function(mode)
  {
    var d = Control.device; // alias
    
    if (mode != d.widget.defaultMode && mode!='docked')
    {
      opera.postError('Unsupported mode attempted: ' + mode)
      return;
    }
    
    if (d.widget.mode == mode)
    {
      return; // nothing to change
    }
    
    if (mode == 'docked')
    {
      if (!d.widget.dockable)
      {
        return; // widget does not implement 'docked'
      }
      emulator.dockWidget();
    }
    else
    {
      emulator.undockWidget();
    }
    
    // avoid flicker
    //emulator.iframe.style.visibility = 'hidden';
    //g('dockBlocker').style.display = 'none';
    //showWidgetWait();
    
  },
  
  /**
   * Change the widget mode
   *
   * <p>This function will change the mode
   * the widget is running in, currently
   * supports 'widget' and 'docked' modes.</p>
   *
   * @param {string} mode The new mode to switch to
   */
  
  toggleDocked: function()
  {
    emulator.changeWidgetMode( Control.device.widget.mode == 'docked'
                              ?  Control.device.widget.defaultMode : 'docked');
  },
  
  setDefaultWidgetMode: function(mode)
  {
    var d = Control.device;
    if (d.widget.mode == d.widget.defaultMode)
    {
      d.widget.mode = mode;
      emulator.window.widget.widgetMode = mode;
    }
    d.widget.defaultMode = mode;
  },
  
  applyChromeSize: function()
  {
    var chrome = Control.device.state.chrome;
    
    g('screen-chrome-top').style.height    = chrome[0] + 'px';
    g('screen-chrome-right').style.width   = chrome[1] + 'px';
    g('screen-chrome-bottom').style.height = chrome[2] + 'px';
    g('screen-chrome-left').style.width    = chrome[3] + 'px';
    
    // left and right toolbars need correct positioning
    g('screen-chrome-right').style.top = //
    g('screen-chrome-left') .style.top = chrome[0] + 'px';
    g('screen-chrome-right').style.height = //
    g('screen-chrome-left') .style.height = (Control.device.screen.height - chrome[0] - chrome[2]) + 'px';
    
    g('screen-chrome').style.visibility = 'visible';
  },

  /**
   *
   * Changes the dimensions of the screen of a device during widget runtime.
   * Dispatches necessary resolution event and moves widget to new position etc.
   *
   * @param width {Integer} Optional width of new screen size
   * @param height {Integer} Optional height of new screen size
   *
   */
  
  setScreenSize: function(width, height)
  {
    // save new dimensions to control
    var d = Control.device;
    
    if (!width)
    {
      width = d.screen.width;
    }
    if (!height)
    {
      height = d.screen.height
    }
    
    d.screen.width = width;
    d.screen.height = height;
    
    d.media.features['device-width'] = width;
    d.media.features['device-height'] = height;
    d.media.features['device-aspect-ratio'] = width/height;
    
    if (d.widget.began && d.widget.mode != 'docked') // docked mode is oblivious to screen changes
    {
      emulator.window.widget.dispatchEvent({
        type: 'resolution',
        width: d.screen.availWidth,
        height: d.screen.availHeight
      });
    }
    
    emulator.updateAvailSize();
    emulator.moveWidgetTo(d.widget.left, d.widget.top);
    
    // makes sure widget isn't hanging off the screen
    if (d.widget.began)
    {
      emulator.applyWidgetDimensions();
    }
    
    ui.setScreenSize(width, height);
    ui.resizeWindow();
    
    window.scrollbars.fix(); // true causes full scrollbar reset
  },
  
  /**
   * Causes the screen to rotate.
   * Should only be called if the device actually allows this.
   */
  
  rotate: function()
  {
    emulator.setScreenSize(Control.device.screen.height, Control.device.screen.width);
    
    //opera.postError(Control.device.settings.dock + '\n'+Control.device.settings.rotatable.dock);
    
    var d = Control.device;
    if (d.widget.began && d.widget.mode=='docked' && d.settings.dock && d.settings.rotatable.dock &&
        (d.settings.dock[0]!=d.settings.rotatable.dock[0] ||
         d.settings.dock[1]!=d.settings.rotatable.dock[1])
    )
    {
      
      emulator.applyDockDimensions();
      
      // dock size has changed, reflect it
      emulator.window.widget.dispatchEvent({
        type: 'resolution',
        width: d.state.dock[0],
        height: d.state.dock[1]
      });
      
    }
  },
  
  /**
  * Sets the device's orientation
  * This differs from .rotate since that is a toggle, whereas this specifically
  * sets the rotation.
  * It can also disable the rotate button.
  *
  * @param orientation {String} 'portrait', 'landscape' or 'auto' (empty string acts as auto), case sensitive
  * @param mode {Boolean} Disables the rotate screen button if true
  */
  
  setScreenOrientation: function(orientation, disableRotateButton)
  {
    if (orientation && orientation != 'auto') // if 'auto' then do nothing (except perhaps re-enable the rotate button below)
    {
      if (orientation != 'portrait' && orientation != 'landscape')
      {
        throw new Error('Unknown orientation specified: '+orientation);
      }
      
      if (!Control.device.settings.rotatable)
      {
        // this is not serious enough to throw an error
        opera.postError('Warning, widget has requested '+orientation+' orientation but the device does not allow rotating the screen.');
        return;
      }
      
      // do the rotate, boolean logic handles both portrait and landscape conditions at once
      if ((Control.device.screen.width < Control.device.screen.height) != (orientation == 'portrait'))
      {
        Control.rotateDevice();
      }
    }
    document.getElementById('rotate').disabled = !!disableRotateButton; // force to boolean
  },
  
  closeWidget: function()
  {
    pluginHandler.closeAllPlugins();
    
    g('device').removeClass('running');
    
    if (emulator._maintenanceInterval)
    {
      clearInterval(emulator._maintenanceInterval);
      emulator._maintenanceInterval = 0;
    }
    
    ui.hideNotification();
    
    ui.dragHandler.disable();
    
    emulator.window = null;
    
    g('dockBlocker').style.display = 'none';
    g('device').removeClass('docked');
    g('frameWrapper').innerHTML = '';
  },
  
  _maintenanceInterval: 0,
  
  /**
   * A function which is to run periodically to ensure that:
   *
   * 1. Changes to the title element during runtime are reflected
   * 2. Keep running the media query matcher in case things change that
   *    are not caught
   */
  
  maintenance: function()
  {
    if (!emulator.window || !emulator.window.document)
    {
      // widget has seized running, no need to keep maintenence
      clearInterval(emulator._maintenanceInterval);
      emulator._maintenanceInterval = 0;
      return;
    }
    
    var str = emulator.window.document.title || '';
    var titleEle = g('info-title');
    if (str != titleEle.textContent)
    {
      docTitle = str;
      titleEle.textContent = str;
    }
    
    Control.device.media.matchDocument(emulator.window.document);
  },
  
  analyseWidget: function(xml)
  {    
    var configXML = null;
    var width = null;
    var height = null;
    
    // any changes to the config.xml since last read will be shown in the manager
    manager.updateWidget(Control.device.widget.locationPath, xml);
    
    try {
      configXML = xml.getElementsByTagName('widget')[0];
    }
    catch(e){
      Control.failWidget('config');
      return;
    }
    
    if (!configXML)
    {
      Control.failWidget('config');
      return;
    }
    
    if (emulator.isWebserverRequested(xml))
    {
      Control.failWidget('webserver');
      return;
    }
    
    var dockable = (configXML.getAttribute('dockable')||'').toLowerCase()
    
    // fixme: dockable="yes" is not allowed per spec. Added as workaround until bug is fixed (runeh)
    Control.device.widget.dockable = Control.device.settings.dock &&
                                     Control.device.settings.dock[0] &&
                                     Control.device.settings.dock[1] &&
                                     (dockable=='1' ||
                                      dockable=='yes' ||
                                      dockable=='true' ||
                                      dockable=='dockable');
    
    if (Control.device.widget.dockable)
    {
      g('screen').addClass('dockable');
    }
    else
    {
      g('screen').removeClass('dockable');
    }
    
    function stripInt(str)
    {
      return parseInt(str.replace(/^\s+0*|\s+$/g,''));
    }
    
    try { width  = stripInt(configXML.getElementsByTagName('width' )[0].firstChild.nodeValue); } catch ( e ) { }
    try { height = stripInt(configXML.getElementsByTagName('height')[0].firstChild.nodeValue); } catch ( e ) { }
    
    // default values
    if ( !width  || width  <= 0) width  = 200;
    if ( !height || height <= 0) height = 150;
    
    Control.device.widget.mode = 'widget';
    Control.device.widget.defaultMode = 'widget';
    
    // record new size
    emulator.updateAvailSize();
    emulator.resizeWidgetTo(width, height);
    
    // set widgetfile
    var widgetfile = "";
    
    try {
      widgetfile = configXML.getElementsByTagName('widgetfile')[0].firstChild.nodeValue;
    }
    catch(e){}
    
    Control.device.widget.widgetfile = widgetfile || 'index.html';
    
    // set icon
    
    try {
      ui.setIcon(configXML.getElementsByTagName('icon')[0].firstChild.nodeValue);
    }
    catch (e)
    {
      ui.setIcon(null)
    }
    
    var d = Control.device;
    
    d.widget.configXML = configXML;
    
    d.widget.opened = true;
    d.widget.loaded = false;
    
    emulator.runWidget();
    
  },
  
  runWidget: function()
  {
    g('desktop').style.display = '';
    g('screen-content').style.display = 'block';
    g('info-program').getElementsByTagName('span')[0].innerHTML = '';
    g('info-program').style.display = 'block';
    g('screen-tools').style.display = 'block';
    
    var d = Control.device;
    
    emulator.moveWidgetTo(
      Math.round( ( d.screen.availWidth  - d.widget.width  ) / 2),
      Math.round( ( d.screen.availHeight - d.widget.height ) / 2)
    );
    
    d.widget.scrollTop = 0;
    d.widget.scrollLeft = 0;
    
    d.widget.storage = 0;
    for (var i in d.widget.preferences)
    {
      if (typeof d.widget.preferences[i] == 'string')
      {
        d.widget.storage += (i+'').length + (d.widget.preferences[i]+'').length;
      }
    }
    
    g('frameWrapper').innerHTML = '\
      <iframe id="iframe" scroll="no" style="visibility:hidden;top:'+d.widget.top+'px;left:'+d.widget.left+'px;" src="'+Control.device.widget.launchPath+Control.device.widget.widgetfile+'" width="'+Control.device.widget.width+'" height="'+Control.device.widget.height+'">\
      <\/iframe>\
    ';
    
    if (!g('iframe').contentDocument)
    {
      Control.failWidget('index');
      return;
    }
    
    /* Hack to detect when Opera can't locate the widgetfile
      but pretends to find a blank document instead.
      Not a nice solution but works for now */
    try {
      if (g('iframe').contentDocument.documentElement.innerHTML == '<HEAD></HEAD><BODY></BODY>')
      {
        throw new Error();
      }
    }
    catch(e)
    {
      Control.failWidget('index');
      return;
    }
    
    g('iframe').addEventListener('load', function()
    {
      if (!d.widget.began && !d.widget.loaded)
      {
        Control.failWidget('index');
      }
    }, false);
    
    ui.updateStatus();
  },
  
  begin: function(frameWin) // creates the wrappers and such
  {
    if (Control.device.widget.began)
    {
      if (frameWin != emulator.window)
      {
        // widget probably called location.reload(); or something similar
        // we restart the emulation process
        Control.device.widget.began = false;
        Control.device.widget.loaded = false;
      }
      else
      {
        // widget has already started emulating (widget probably had incantation twice)
        // don't set continue with the function
        return;
      }
    }
    
    emulator.window = emulator.window = frameWin; // expose in larger scope (not globally though)
    
    emulator.setScreenSize();
    
    // if the widget reloaded itself with location.reload then we need
    // to stop the plugins before they are started again
    pluginHandler.closeAllPlugins();
    
    Control.device.widget.began = true;
    
    emulator.applyChromeSize();
    
    if (!emulator.window)
    {
      opera.postError('Send a reference to the window object as the first argument to this.parent.begin.');
      return;
    }
    
    // add css to make it undraggable, and remove scrollbars
    emulator.window.document.write('<style type="text/css">'+
      'html{overflow:hidden;position:absolute;top:0;left:0;right:0;bottom:0;-apple-dashboard-region:dashboard-region(control rectangle);}'+
      '</style>');
    
    emulator.iframe = g('iframe');
    
    emulator.window.addEventListener('DOMContentLoaded',function()
    {
      Control.device.widget.loaded = true;
      
      if (emulator._maintenanceInterval)
      {
        // don't want this running twice as often as it should
        // could potentially happen in some strange edgecase
        clearInterval(emulator._maintenanceInterval);
      }
      
      // widget DOM is now ready for maintenance
      emulator.maintenance(); // do one straight away
      interval = setInterval(emulator.maintenance, 1000); // then every second
      
      // we hide the iframe while it is loading
      emulator.iframe.style.visibility = 'visible';
      
    }, true);
    
    // set user agent
    if (Control.device.settings.useragent)
    {
      emulator.window.navigator.userAgent = Control.device.settings.useragent;
    }
    
    /**
     * Shows the widget after 1ms has passed.
     * 
     * Used by changeWidgetMode. Necessary to avoid
     * flickering content when switching to 'dock' then
     * 'widget' mode quicky.
     */
    
    var showWidgetWait = (function()
    {
      var timeout = 0;
      
      function show()
      {
        emulator.iframe.style.visibility = 'visible';
        g('dockBlocker').style.display = Control.device.widget.mode == 'docked' ? 'block' : 'none';
        timeout = 0;
      }
      
      return function()
      {
        if (!timeout)
        {
          timeout = setTimeout(show,1);
        }
      }
    })();
    
    // remove reference to the parent window
    emulator.window.parent = emulator.window;
    emulator.window.top = emulator.window;
    
    // change the screen size and properties
    emulator.window.screen = Control.device.screen;
    
    emulator.window.screenLeft = Control.device.widget.left;
    emulator.window.screenTop = Control.device.widget.top;
    emulator.window.innerWidth = Control.device.widget.width;
    emulator.window.innerHeight = Control.device.widget.height;
    
    /* supply a widget object, constructor is at the bottom of this script */
    emulator.window.widget = new Widget();
    
    /* over-ride the close function */
    emulator.window.close = function()
    {
      Control.closeWidget();
    };
    
    emulator.window.resizeTo = function(valueWidth, valueHeight)
    {
      emulator.resizeWidgetTo(valueWidth, valueHeight);
      emulator.applyWidgetDimensions();
    };
    
    emulator.window.resizeBy = function(deltaWidth, deltaHeight)
    {
      emulator.resizeWidgetTo(Control.device.widget.width + deltaWidth, Control.device.widget.height + deltaHeight);
      emulator.applyWidgetDimensions();
    };
    
    emulator.window.moveTo = function(valueLeft, valueTop)
    {
      emulator.moveWidgetTo(valueLeft, valueTop);
      emulator.applyWidgetDimensions();
    };
    
    emulator.window.moveBy = function(deltaLeft, deltaTop)
    {    
      emulator.moveTo(Control.device.widget.top + deltaTop, Control.device.widget.left + deltaLeft);
      emulator.applyWidgetDimensions();
    };
    
    emulator.window.document.documentElement.addEventListener('DOMNodeInserted',function(e)
    {
      // if the dom changes we need to re-run the media query emulator
      Control.device.media.matchDocument(emulator.window.document);
    }, false);
    
    /*
      Any click occuring in the widget needs to be handled.
      3 things are done:
      
      1. If the widget is docked, clicking undocks it
      2. The parent needs to receive the click as well for various reasons
      3. If a link was clicked we must ensure it has target=_blank set
    */
    
    emulator.window.document.addEventListener('click', function(e)
    {
      if (Control.device.widget.mode == 'docked')
      {
        emulator.toggleDocked(); // clicking docked widget should 'undock'
      }
      
      // when a click occurs on the iframe the parent window doesn't receive it
      // we must propagate the event ourselves to the main window otherwise
      // for instance the deviceSelector may remain hanging open when it shouldn't
      var evt = document.createEvent("MouseEvents");
      // most of the values below don't really matter, just make sure it is 'click'
      evt.initMouseEvent("click", true, true, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
      document.dispatchEvent(evt);
      
      /*
          Stop anchor tags causing the iframe to change location
      */
      
      var ele = e.target;
      var href = '';
      
      while (ele)
      {
        // we have to check the tagName instead of HTMLAnchorElement
        // because that object's prototype is different between the
        // parent and the iframe
        if (ele && 'a' == (ele.tagName||'').toLowerCase()
            && !ele.getAttribute('target')
            && (href = ele.getAttribute('href'))
            && href.indexOf('#') // not *starting* with #
            && href.indexOf('javascript:')) // not *starting* with javascript:
        {
          ele.setAttribute('target', '_blank');
        }
        ele = ele.parentNode;
      }
      
    }, false);
    
    /*
      Stop form submitions from reloading the iframe
    */
    
    emulator.window.document.addEventListener('submit',function(evt)
    {
      try {           
        /*
          evt.target.target is the target attribute of the form.
          If it is present we should not abort the form submission
          as it will naturally open in a new window or an iframe
          
          If the form has a reserved target*, and this target leads to
          intrisically replacing the topmost document in the widget,
          in effect replacing the widget, submitting the form should
          fail silently
          
          * Opera treats reserved targets as case-insensitive
          http://www.w3.org/TR/html401/types.html#type-frame-target
        */
        
        var targetString = (evt.target.target||'')+'' // ensure it is a string
        
        if (!targetString)
        {
          evt.preventDefault();
        }
        else
        {
          targetString = targetString.toLowerCase();
          
          if (
            targetStringLC == '_self' ||
            targetStringLC == '_top' ||
            targetStringLC == '_parent')
          {
            opera.postError('A form must must not use target="'+targetString+'".');
            evt.preventDefault();
          }
          else
          {
            if (!emulator.window.frames[targetString] &&
                ((evt.target.method+'')+'').toLowerCase() == 'post')
            {
              opera.postError('A form must not POST to a new window.');
              evt.preventDefault();
            }
          }
        }
      }
      catch(ignore)
      {
        // nothing to do if there is an error
      }
      
    }, false)
    
    ui.dragHandler.enable(emulator.window);
    scrollbars.fix(false); // this allows us to add scroll events to the window
    
    g('device').addClass('running');
    g('plugin-ui').innerHTML = '';
    Control.startPlugins(emulator.window);
    
    ui.resizeWindow();
  }
  
};



/**
 * Widget object
 * Creates a widget object to be assigned to emulator.window.widget.
 *
 * @constructor
 */

function Widget()
{
  this.widgetMode = 'widget';
  this.__eventListeners = {}; /* private */
}

Widget.prototype = {
  
  toString: function()
  {
    return '[object Widget]';
  },
  
  addEventListener: function(evt, funct)
  {
    evt = evt.toLowerCase();
    if (!this.__eventListeners.hasOwnProperty(evt))
    {
      this.__eventListeners[evt] = [];
    }
    this.__eventListeners[evt].push(funct);
  },
  
  removeEventListener: function(evt, funct)
  {
    evt = ((evt||'')+'').toLowerCase();
    if (!this.__eventListeners.hasOwnProperty(evt)) { return; }
    var arr = this.__eventListeners[evt];
    for (var i=0; i<arr.length; i++)
    {
      if (arr[i] == funct)
      {
        arr.splice(i,1);
        i--;
      }
    }
  },
  
  dispatchEvent: function(event)
  {
    var type = ((event.type||'')+'').toLowerCase();
    if (!this.__eventListeners.hasOwnProperty(type)){return;}
    var arr = this.__eventListeners[type];
    
    for (var i=0; i<arr.length; i++)
    {
      arr[i].call(emulator.window.widget, event);
    }
  },
  
  preferenceForKey: function(key)
  {
    return Control.device.widget.preferences[key] || '';
  },
  
  setPreferenceForKey: function(pref, key)
  {
    pref = '' + pref, key = '' + key; // make them strings
    var oldPref = Control.device.widget.preferences[key];
    var storage;
    if (pref)
    {
      storage = Control.device.widget.storage + (oldPref ? pref.length - oldPref.length : pref.length + key.length);
      if (storage > Control.device.settings.storage)
      {
        throw new Error('Widget preferences have exceeded the maximum volume.');
      }
      Control.device.widget.storage += oldPref ? pref.length - oldPref.length : pref.length + key.length;
      Control.device.widget.preferences[key] = pref.toString();
    }
    else
    {
      delete Control.device.widget.preferences[key];
    }
    ui.updateStatus();
    Control.savePreferences();
  },
  
  openURL: function()
  {
    widget.openURL.apply(widget, arguments);
  },
  
  show: function()
  {
    g('screen').removeClass('widgetMinimised');
  },
  
  hide : function()
  {
    g('screen').addClass('widgetMinimised');
  },
  
  showNotification : function(str, callback)
  {
    ui.showNotification(str, callback);
  },
  
  getAttention : (function()
  {
    var count = 0;
    var MAX = 12;
    var interval = 0;

    function flash()
    {
      count++;
      if (count == MAX)
      {
        clearInterval(interval);
        interval = 0;
        g('info-program').removeClass('flashing');
      }
      else
      {
        g('info-program').toggleClass('flashing')
      }
    }
    
    return function()
    {
      count = 0;
      if (!interval)
      {
        interval = setInterval(flash, 300);
      }
    };
  })()
};