/**
 * @fileoverview
 *
 * <p>This file handles *everything* to do with fileio.
 * No other files should access the fileio apis.
 * Other files may however pass around strings containing mounpoint urls.</p>
 *
 * <p>Methods include reading files, browsing for plugins/devices/widgets and
 * deleting these.</p>
 */

var fileio = {
  
  // each time we browse for a widget we give it a mountpoint name 'widget1', 'widget2'... etc.
  widgetCount: parseInt(widget.preferenceForKey('widgetCount')) || 1,
  
  // storage mountpoint
  storage: opera.io.filesystem.mountSystemDirectory('storage', 'storage'),
  // application mountpoint
  application: opera.io.filesystem.mountSystemDirectory('application', 'application'),
  
  /**
   * When a plugin is asked to be deleted it is deleted the next time the widget reloads.
   * This seems to be the only way to do it reliably.
   * To do this they are added to the following array and removed on next reload.
   */
  
  pluginsPendingDeletion: widget.preferenceForKey('pluginsPendingDeletion') || [],
  
  // temporary mountpoint names used for browse dialoges for plugins and devices
  BROWSE_PLUGIN_MOUNTPOINT_NAME: 'BrowseForPlugin',
  BROWSE_DEVICES_MOUNTPOINT_NAME: 'BrowseForDevices',
  
  init: function()
  {
    //opera.postError(this.widgetCount);
    // no need for init yet, good to keep this way
  },
  
  /**
   * Launches a browse dialogue to load a widget.
   */
  
  browseForWidget: function()
  {
    try {
      opera.io.filesystem.browseForDirectory('widget'+fileio.widgetCount, 'downloads', fileio.browseForWidgetCallback, true);
    }
    catch (e)
    {
      ui.alert("Error", "Error launching browse dialogue.");
    }
  },
  
  /**
   * Callback from browseForWidget.
   * Performs some validation on the file then adds it to the widget manager
   * if all is successful.
   *
   * @param dir {File} A File object that was browsed to.
   */
  
  browseForWidgetCallback: function(dir)
  {
    var file = null;
    var text = '';
    var xml = null;
    
    if (!dir || !dir.exists || !dir.isDirectory)
    {
      // die silently, user probably cancelled the operation
      fileio.removeMountPoint('widget'+fileio.widgetCount);
      return;
    }
    
    try {
      file = dir.resolve('config.xml');
      
      if (!file.exists)
      {
        ui.alert("Widget could not be imported", "No config.xml file found.")
        throw new Error();
      }
      
      text = fileio.getFileContents(file);
      
      xml = (new DOMParser()).parseFromString(text, "text/xml");
      
      if (!xml.getElementsByTagName('widget')[0])
      {
        ui.alert("Widget could not be imported", "No <widget> element found in config.xml file.");
        throw new Error();
      }
      
      for (var i=0; i<manager.widgets.length; i++)
      {
        if (manager.widgets[i].nativePath == dir.nativePath)
        {
          ui.alert("Widget Already Added", "The widget \""+manager.widgets[i].name+"\" has already been added.");
          throw new Error();
        }
      }
    }
    catch(err)
    {
      fileio.removeMountPoint('widget'+fileio.widgetCount);
      return;
    }
    
    fileio.widgetCount++;
    widget.setPreferenceForKey(fileio.widgetCount, 'widgetCount');
    
    manager.addWidget(dir.path, dir.nativePath);
  },
  
  /**
   * Launches a browse dialogue to load a device definition file.
   */
  
  browseForDevices: function()
  {
    try {
      opera.io.filesystem.browseForFile(fileio.BROWSE_DEVICES_MOUNTPOINT_NAME, '', fileio.browseForDevicesCallback, false);
    }
    catch (e)
    {
      opera.postError(e);
      ui.alert("Error", "Error launching browse dialogue.");
    }
  },
  
  /**
   * Callback from browseForDevices.
   * Performs some validation on the file then adds the devices found to Control
   * if all is successful.
   *
   * @param dir {File} A File object that was browsed to.
   */
  
  browseForDevicesCallback: function(file)
  {
    var contents = "";
    var obj = null;
    
    if (!file || !file.exists || !file.isFile)
    {
      // die silently, user probably cancelled the operation
      return;
    }
    
    try {
      contents = fileio.getFileContents(file);
      
      obj = contents && JSON.parse(contents);
      
      if (typeof obj != 'object')
      {
        throw new Error();
      }
      
      fileio.removeMountPoint(fileio.BROWSE_DEVICES_MOUNTPOINT_NAME);
    }
    catch(e)
    {
      ui.alert('Error Loading Devices', 'The device definition file is malformed.');
      fileio.removeMountPoint(fileio.BROWSE_DEVICES_MOUNTPOINT_NAME);
      return;
    }
    
    Control.importDevices(obj);
  },
  
  /**
   * Launches a browse dialogue to load a plugin.
   */
  
  browseForPlugin: function()
  {
    try {
      opera.io.filesystem.browseForFile(fileio.BROWSE_PLUGIN_MOUNTPOINT_NAME, '', fileio.browseForPluginCallback, false);
    }
    catch (e)
    {
      ui.alert('Error', 'An error occured launching the browse dialogue.\
                         Please ensure that your browser and emulator version are up to date.');
    }
  },
  
  /**
   * Callback from browseForPlugin.
   * Performs some validation on the plugin (including possibly user verification
   * to overwrite an existing plugin). Calls fileio.makeStorageCopyOfPlugin
   * if all is successful.
   *
   * @param dir {File} A File object that was browsed to.
   */

  browseForPluginCallback: function(file)
  {
    var xmlFile = null;
    var xmlContents = null;
    var pluginName = '';
    
    if (!file || !file.exists)
    {
      // die silently, user probably cancelled the operation
      return;
    }
    
    if (!file.isArchive)
    {
      ui.alert('Invalid Plugin', 'The plugin could not be read, please ensure it is a valid zip file.');
      fileio.removeMountPoint(fileio.BROWSE_PLUGIN_MOUNTPOINT_NAME);
      return;
    }
    
    xmlFile = file.resolve('plugin.xml');
    
    if (!xmlFile.exists)
    {
      ui.alert('Invalid Plugin', 'The plugin does not contain a plugin.xml file.');
      fileio.removeMountPoint(fileio.BROWSE_PLUGIN_MOUNTPOINT_NAME);
      return;
    }
    
    xmlContents = fileio.getFileXML(xmlFile);
    
    if (!xmlContents || !xmlContents.firstChild || xmlContents.firstChild.tagName != 'plugin')
    {
      ui.alert('Invalid Plugin', 'The plugin.xml file is malformed.');
      fileio.removeMountPoint(fileio.BROWSE_PLUGIN_MOUNTPOINT_NAME);
      return;
    }
    
    pluginName = (xmlContents.firstChild.getAttribute('name') || '').replace(/^\s+|\s+$/g, '');
    if (!pluginName)
    {
      ui.alert('Invalid Plugin', 'The plugin.xml file contains no name on the <plugin> element.');
      fileio.removeMountPoint(fileio.BROWSE_PLUGIN_MOUNTPOINT_NAME);
      return;
    }
    
    if (pluginHandler.plugins[pluginName])
    {
      ui.confirm('Replace Existing Plugin?',
        [
          ['p',
            'You already have a plugin with the name: ',
            ['b', pluginName]
          ],
          ['p',
            'Existing version: ', ['b', pluginHandler.plugins[pluginName].version],
            ['br'],
            'Your version: ', ['b', xmlContents.firstChild.getAttribute('version')]
          ]
        ],
        null,
        function(ok)
        {
          if (!ok)
          {
            fileio.removeMountPoint(fileio.BROWSE_PLUGIN_MOUNTPOINT_NAME);
            return;
          }
          Control.deletePlugin(pluginName, true);
          fileio.makeStorageCopyOfPlugin(file);
        }
      );
      
      return;
    }
    
    fileio.makeStorageCopyOfPlugin(file);
  },
  
  /**
   * Takes a plugin zip and copies it to storage. Makes no validation of the
   * archive file. Calls Control.addPlugin with the new path of the plugin copy.
   * 
   * @param file {File} The zip archive of the plugin to make a local storage copy of
   */
  
  makeStorageCopyOfPlugin: function(file)
  {
    var newName = '';
    var newFile = null;
    
    // ensure we pick a file name that isn't already used
    // it doesn't matter what the name actually is
    while (!newFile || newFile.exists)
    {
      newName = 'plugin'+Math.floor(100000000*Math.random())+'.zip';
      newFile = fileio.storage.resolve('plugins').resolve(newName);
    }
    
    file.copyTo(newFile);
    
    fileio.removeMountPoint(fileio.BROWSE_PLUGIN_MOUNTPOINT_NAME);
    
    Control.addPlugin(newFile.path+'/');
  },
  
  /**
   * Shorthand way of unmounting a mountpoint.
   *
   * @param name {String} The name of the mouintpoint to remove.
   */
  
  removeMountPoint: function(name)
  {
    try {
      opera.io.filesystem.removeMountPoint(opera.io.filesystem.mountPoints.resolve(name));
    }
    catch(e)
    {
      opera.postError('Could not remove mountpoint: '+name+'\n\n'+e);
    }
  },
  
  /**
   * Removes a widget's mountpoint.
   * Similar to above.
   * Do I need this any more?? [todo]
   */
  
  removeWidget: function(name)
  {
    try {
      opera.io.filesystem.removeMountPoint(opera.io.filesystem.mountPoints.resolve(name));
    }
    catch(e)
    {
      return false;
    }
    return true;
  },
  
  /**
   * Gets the list of all internal widgets.
   *
   * @returns {Array} An array of strings representing the mountpoint urls of the widgets.
   */
  
  getInternalWidgetsList: function()
  {
    var list = [];
    var app = fileio.application.resolve('widgets');
    
    app.refresh();
    
    for (var i=0; i<app.length; i++)
    {
      if (app[i].isDirectory && app[i].name != '.svn' // DEBUG
      )
      {
        list[list.length] = 'mountpoint://application/widgets/'+app[i].name;
      }
    }
    
    return list;
  },
  
  /**
   * Gets the list of all plugins both prepackaged and subsequently imported ones.
   *
   * @returns {Array} An array of strings representing the mountpoint urls of the plugins.
   */
  
  getPluginsList: function()
  {
    var list = fileio.getPluginsSublist(fileio.storage.resolve('emulator').resolve('plugins'));
    return list.concat(fileio.getPluginsSublist(fileio.storage.resolve('plugins')))
  },
  
  /**
   * Performs the actual scanning of a directory to find plugins.
   * Better as a seperate function as it can be called once for prepackaged plugins
   * and again for the subsequently imported plugins.
   *
   * @param dir {File} A folder to search for plugins (either zips or folders).
   * @returns {Array} An array of strings representing the mountpoint urls of the plugins.
   */
  
  getPluginsSublist: function(dir)
  {
    var list = [];
    var index = 0;
    
    dir.refresh();
    
    for (var i=0; i<dir.length; i++)
    {
      if ((dir[i].isDirectory || dir[i].isArchive)
          && dir[i].name != '.svn' // DEBUG
      )
      {
        if (dir[i].isArchive && 0 <= (index = fileio.pluginsPendingDeletion.indexOf(dir[i].path+'/')))
        {
          // file is pending deletion, try to delete it, even if we can't we don't offer the
          // plugin to the user
          if (dir[i].deleteFile(dir[i]))
          {
            // file successfully deleted remove from pending deletion list
            fileio.pluginsPendingDeletion.splice(index, 1);
            widget.setPreferenceForKey(fileio.pluginsPendingDeletion, 'pluginsPendingDeletion');
          }
        }
        else
        {        
          list[list.length] = dir[i].path + '/';
        }
      }
    }
    
    return list;
  },
  
  /**
   * Deletes the local storage copy of the plugin.
   *
   * @param pluginName {String} The name of the plugin to remove.
   * @returns {Boolean} True if the plugin was deleted sucessfully, else false
   */
  
  deletePlugin: function(pluginName)
  {
    var plugin = pluginHandler.plugins[pluginName];
    
    fileio.pluginsPendingDeletion.push(plugin.path);
    widget.setPreferenceForKey(fileio.pluginsPendingDeletion, 'pluginsPendingDeletion');
  },
  
  /**
   * Deletes a file given its path
   *
   * @param path {String} The path of the file (including mountpoint://...)
   * @returns {Boolean} True if the file was deleted sucessfully, else false
   */
  
  deleteFile: function(path)
  {
    var file = opera.io.filesystem.mountPoints.resolve(path);
    
    return file.deleteFile(file);
  },
  
  /**
   * Reads the string contents of a file.
   *
   * @param path {String} The mouintpoint url of the file to read.
   * @returns {String} The string contents of the file.
   *
   */
  
  getFileContents: function(path)
  {
    var file = opera.io.filesystem.mountPoints.resolve(path);
    var stream = null;
    var contents = "";
    
    try {
      stream = file.open('', 'r');
    }
    catch(e)
    {
      return null;
    }
    
    while (!stream.eof)
    {
      contents += (contents ? stream.newLine : '') + stream.readLine();
    }
    
    stream.close();
    
    return contents;
  },
  
  /**
   * Reads and parses the contents of an xml file.
   * Without a callback function the operating is synchronous.
   * 
   * @param path {String} The mouintpoint url of the xml file to read.
   * @param callback {Function} Optional function to call once reading and parsing is completed.
   */
  
  getFileXML: function(path, callback)
  {
    var contents = fileio.getFileContents(path);
    var xml = null;
    
    if (typeof contents == 'string')
    {
      xml = new DOMParser().parseFromString(contents, "text/xml");
    }
    
    if (callback)
    {
      callback(xml);
      return null;
    }
    else
    {
      return xml;
    }
  },
  
  /**
   * Copies the widget to a storage location where it can be launched and
   * modified (by inserting the special script).
   *
   * @param path {String} The path (including mountpoint) to the widget.
   * @returns The new launchable location of the widget.
   */
  
  prepareWidgetForLaunch: function(path)
  {
    var storage = fileio.storage;
    var sandbox = storage.resolve('sandbox');
    if (sandbox.exists)
    {
      storage.deleteDirectory(sandbox, true);
    }
    if (!sandbox.exists)
    {
      storage.createDirectory(sandbox);
    }
    
    var from = opera.io.filesystem.mountPoints.resolve(path);
    copyFiles(from, sandbox);
    
    var xml = fileio.getFileXML(sandbox.resolve('config.xml'));
    
    var widgetfile = '';
    try {
      widgetfile = xml.getElementsByTagName('widgetfile')[0].firstChild.nodeValue || 'index.html';
    }
    catch(e)
    {
      widgetfile = 'index.html';
    }
    
    if (!fileio.insertScript(sandbox.resolve(widgetfile)))
    {
      return null;
    }
    
    return 'mountpoint://storage/sandbox/';
  },
  
  /**
   * Inserts the special script to start the emulation process.
   *
   * @param file {File} The html file to insert the script.
   * @returns {Boolean} True if the script was inserted, else false.
   */
  
  insertScript: function(file)
  {
    var INCANTATION = '<script>parent.emulator.begin(this)</script>';
    var stream = null;
    var contents = "";
    
    try {
      stream = file.open('', 'r');
    }
    catch (e)
    {
      return false;
    }
    
    
    while (!stream.eof)
    {
      contents += stream.readLine() + (stream.eof ? '' : stream.newLine);
    }
    
    stream.close();
    
    /* This regexp will insert the incantation in the correct place,
      that is after any <!doctype ..>: \s*<![^>]>\s*
      or after any <?xml ..>         : \s*<\?[^\?]*\?\s*
    */
    
    contents = contents.replace(/^(\s*<\?[^\?]*\?>\s*)?(\s*<![^>]*>\s*)?([^]*)$/, '$1$2'+INCANTATION+'$3')
    
    // open with 'w': clears the file contents so we can write it afresh
    stream = file.open('', 'w');
    stream.write(contents);
    stream.close();
    
    return true;
  }
};