/**
 *
 * @fileoverview
 *
 * <p>Handles the listing of available widgets in the 'widgets' directory</p>
 * 
 */



var manager = {
    
    widgets: [],
    
    pageOffset: 0,
    pageCount: null,
    pageFits: null, // how many widgets fit on each page
    
    init: function()
    {
        // add internal widgets
        var internal = fileio.getInternalWidgetsList();
        for (var i=0; i<internal.length; i++)
        {
            this.widgets.push(new WidgetItem(internal[i], null));
        }
        
        // add external widgets
        var name = '';
        var nativePath = '';
        for (var i=0; i<opera.io.filesystem.mountPoints.length; i++)
        {
            name = opera.io.filesystem.mountPoints[i].name;
            nativePath = opera.io.filesystem.mountPoints[i].nativePath;
            if (name != 'storage' && name != 'application')
            {
                this.widgets.push(new WidgetItem('mountpoint://'+name, nativePath));
            }
        }
        
        // show them all
        this.listWidgets();
        
        g('widgets-add').addEventListener('click', fileio.browseForWidget, false);
        
        g('debug-add').onclick = function()
        {
            manager.addWidget(manager.widgets[manager.widgets.length-1].locationPath);
        }
    },
    
    /**
     *
     * @param locationPath {String} Location path of widget to get
     * @returns {WidgetItem} The widget if it exists at that path else null.
     */
    
    getWidgetByLocationPath: function(locationPath)
    {
        for (var i=0; i<manager.widgets.length; i++)
        {
            if (manager.widgets[i].locationPath == locationPath)
            {
                return manager.widgets[i];
            }
        }
        return null;
    },
    
    addWidget: function(locationPath, nativePath)
    {
        var widget = new WidgetItem(locationPath, nativePath);
        
        manager.widgets[manager.widgets.length] = widget;
        
        g('desktop-widgets-list').appendChild(widget.li);
        
        manager.resolvePagination();
    },
    
    listWidgets: function()
    {
        var frag = document.createDocumentFragment();
        
        for (var i=0; i<manager.widgets.length; i++)
        {
            frag.appendChild(manager.widgets[i].li);
        }
        
        g('desktop-widgets-list').textContent = '';
        g('desktop-widgets-list').appendChild(frag);
        
        manager.resolvePagination();
    },
    
    updateWidget: function(locationPath, configXML)
    {
        for (var i=0; i<manager.widgets.length; i++)
        {
            if (manager.widgets[i].location == locationPath)
            {
                manager.widgets[i].configXML = configXML;
                manager.widgets[i].analyseCallback(false);
            }
        }
    },
    
    resolvePagination: function()
    {
        var list = g('desktop-widgets-list');
        var pages = g('widgets-pages');
        var items = list.getElementsByTagName('li');
        
        g('desktop-widgets').removeClass('paged');
        
        if (list.scrollHeight <= list.offsetHeight)
        {
            // everything fits on one page, leave it as it is
            return;
        }
        
        // remember the height of an item
        var itemHeight = items[0].offsetHeight;
        
        /* we must show pages */
        g('desktop-widgets').addClass('paged');
        
        // solve how many widgets fit on each page
        manager.pageFits = Math.floor(list.offsetHeight/itemHeight)
        manager.pageCount = Math.ceil(items.length/manager.pageFits);
        manager.pageOffset = Math.min(manager.pageOffset, manager.pageCount-1);
        
        manager.createPageNumbers();
        manager.showPage();
    },
    
    showPage: function()
    {
        var list = g('desktop-widgets-list');
        var items = list.getElementsByTagName('li');
        var showing = list.getElementsByClassName('showing');
        
        for (var i=showing.length-1; i>=0; i--) // count backwards because it is a live list
        {
            showing[i].removeClass('showing');
        }
        for (i = manager.pageOffset*manager.pageFits;
             i < (manager.pageOffset+1)*manager.pageFits && items[i];
             i++
        )
        {
            items[i].addClass('showing');
        }
        
        var pages = g('widgets-pages');
        var chosen = pages.getElementsByClassName('chosen');
        
        for (i=chosen.length-1; i>=0; i--)
        {
            chosen[i].removeClass('chosen');
        }
        pages.getElementsByTagName('li')[manager.pageOffset].addClass('chosen');
        
    },
    
    createPageNumbers: function()
    {
        var self = manager;
        var container = g('widgets-pages');
        var li = null;
        
        container.textContent = ''; // clear it
        
        for (var i=0; i<self.pageCount; i++)
        {
            li = document.createElement('li');
            li.textContent = (i+1); // humans like to count from '1'
            if (i==self.pageOffset)
            {
                li.setAttribute('class', 'chosen');
            }
            li.setAttribute('data-page', i);
            li.addEventListener('click', self.clickPage, false);
            container.appendChild(li);
        }
    },
    
    clickPage: function()
    {
        var page = parseInt(this.getAttribute('data-page'));
        manager.pageOffset = page;
        manager.showPage();
    }
}


/**
 * @constructor
 *
 * An object which represents a widget in the list.
 *
 * @param locationPath {String} Path to the widget (mountpoint://...)
 * @param nativePath {String} Native path to the widget (null if internal)
 */

function WidgetItem(locationPath, nativePath)
{
    this.locationPath = locationPath;
    this.nativePath = nativePath;
    this.configXML = null;
    
    this.name = ''; 
    this.desc = '';
    this.tooltip = '';
    this.icon = '';
    
    this.li = null;
    
    this.createUI();
    this.analyse();
}
WidgetItem.prototype =
{
    /**
     * toString method.
     * Helps for debugging, displays widget's locationPath.
     * @returns {String} useful description
     */
    
    toString: function()
    {
        return '[object WidgetItem: '+this.locationPath+']';
    },
    
    /**
     * Creates the widget's UI
     */
    
    createUI: function()
    {
        this.li = createHTML(['li',
            ['button',
                'type', 'button',
                'data-locationPath', this.locationPath,
                'onclick', this.launchClick,
                ['img',
                    'src', 'emulator_files/img/wgt_icon.png'
                ],
                ['span'],
                ['small']
            ]
        ]);
        
        if (this.nativePath)
        {
            this.li.appendChild(createHTML(['input',
                'type', 'button',
                'class', 'remove',
                'data-locationPath', this.locationPath,
                'onclick', this.removeClick 
            ]));
        }
    },
    
    /**
     * Applies name, description (etc.) properties to the widgetItem's UI
     */
    
    updateUI: function()
    {
        if (this.icon)
        {
            this.li.getElementsByTagName('img')[0].src = this.icon;
        }
        this.li.getElementsByTagName('button')[0].setAttribute('data-tooltip', this.tooltip || '');
        this.li.getElementsByTagName('span')[0].textContent = this.name;
        this.li.getElementsByTagName('small')[0].textContent = this.desc;
    },
    
    /**
     * Removes the widget's UI.
     */
    
    removeUI: function()
    {
        this.li.parentNode.removeChild(this.li);
    },
    
    /**
     * Loads the configXML file of the widget.
     * Applies icons, names etc. to the UI.
     * This is an asynchronous operation, properties are set on analyse continue.
    */
    analyse: function()
    {
        var file = opera.io.filesystem.mountPoints.resolve(this.locationPath).resolve('/config.xml');
        var self = this;
        
        // load the XML file
        fileio.getFileXML(file, this.analyseContinue.bind(this));
    },
    
    /**
     * Continues the analyse from above
     * Applies icons, names etc. to the UI.
     *
     * @param xml {Document} The widget.xml file containing the info needed for the widget listing.
     */
    
    analyseContinue: function(xml)
    {
        this.name = 'No Name';
        this.desc = '';
        this.tooltip = '';
        
        this.configXML = xml;
        
        if (!this.configXML)
        {
            this.name = 'Malformed Widget';
            this.desc = 'config.xml file could not be read'
            this.li.addClass('error');
        }
        else
        {
            this.li.removeClass('error');
            
            // find out the widgetName
            try {
                this.name = this.configXML.getElementsByTagName('widgetname')[0].firstChild.nodeValue;
            }
            catch(ignore){} // remain as 'No name'
            
            // find out the widget description
            try {
                this.desc = this.configXML.getElementsByTagName('description')[0].firstChild.nodeValue;
            }
            catch(ignore){}
            
            // set the icon if it is specified
            try {
                this.icon = this.locationPath+'/'+this.configXML.getElementsByTagName('icon')[0].firstChild.nodeValue;
            }
            catch(ignore){}
            
            this.tooltip = this.nativePath || 'Bundled widget';
        }
        
        this.updateUI();
    },
    
    /**
     * Event handler for clicking the widget's UI list item
     * Loads the widget
     */
    
    launchClick: function(evt)
    {
        var locationPath = this.getAttribute('data-locationPath');
        
        // make sure clicking 'Enter' again doesn't relaunch the widget
        this.blur();
        
        var widgetItem = manager.getWidgetByLocationPath(locationPath);
        
        if (!widgetItem.configXML)
        {
            widgetItem.analyse();
        }
        
        Control.loadWidget(locationPath);
    },
    
    /**
     * Event handler for clicking remove widget
     * Removes the widget
     */
    
    removeClick: function(evt)
    {
        var locationPath = this.getAttribute('data-locationPath');
        
        for (var i=manager.widgets.length-1; i>=0; i--)
        {
            if (manager.widgets[i].locationPath == locationPath)
            {
                manager.widgets[i].removeUI();
                manager.widgets.splice(i,1);
                break;
            }
        }
        manager.resolvePagination();
        
        fileio.removeWidget(locationPath);
    }
};