/**
 * Handles loading and execution of plugins
 * @constructor;
 */

var pluginHandler = (function()
{    
    if (!window.widget) { return; }
    
    /**
     * @constructor
     * @param path {String} The 'mountpoint://...' path of the plugin file/folder
     * @param callback {Function} Called once all scripts have loaded, passes *this* as first argument
     */
    
    function Plugin(path, callback)
    {       
        var self = this;
        
        this.name = '';
        this.group = '';
        this.xml = null;
        this.scope = null;
        this.vars = [];
        this.path = path;
        this.script = "";
        this.author = {
            name: '',
            link: '',
            organisation: ''
        };
        this.description = '';
        this.version = ''; // string, eg. since could be 1.0.1
        
        fileio.getFileXML(path+'plugin.xml', function(xml)
        {
            var author = null;
            var ele = null;
            var root = null;
            
            
            
            try {
                root = xml.firstChild;
                
                if (root.tagName != 'plugin')
                {
                    throw new Error('No plugin element found.');
                }
                
                self.name = xml.firstChild.getAttribute('name');
                self.group = xml.firstChild.getAttribute('group') || '';
                self.version = xml.firstChild.getAttribute('version') || '';
                self.xml = xml.firstChild.cloneNode(true)
                
                // store description
                ele = xml.firstChild.getElementsByTagName('description')[0];
                self.description = ele && ele.firstChild && ele.firstChild.nodeValue || '';
                
                var author = xml.firstChild.getElementsByTagName('author')[0];
                if (author)
                {
                    // store author name
                    ele = author.getElementsByTagName('name')[0];
                    self.author.name = ele && ele.firstChild && ele.firstChild.nodeValue || '';
                    
                    // store author link
                    ele = author.getElementsByTagName('link')[0];
                    self.author.link = ele && ele.firstChild && ele.firstChild.nodeValue || '';
                    
                    // store author organisation
                    ele = author.getElementsByTagName('organization')[0];
                    self.author.organization = ele && ele.firstChild && ele.firstChild.nodeValue || '';
                }
                
                pluginHandler.plugins[self.name] = self;
            }
            catch(err)
            {
                opera.postError('Error loading Emulator Plugin: '+path+'\n'+err)
                setTimeout(callback, 1);
                return;
            }
            self.loadScripts(callback);
        });
    }
    
    Plugin.prototype = {
        /**
        * toString method.
        * Aids in debugging.
        * @returns {string} Useful description of the object
        */
        toString: function()
        {
            return '[object Plugin: '+this.name+']';
        },
        
        /**
         * This preloads the <script> elements.
         * They must be ready before the widget runs.
         *
         * @param callback {Function} Called once all scripts have loaded, passes *this* as first argument
         */
        
        loadScripts: function(callback)
        {
            var script = this.xml.getElementsByTagName('script');
            var path = '';
            var text = '';
            
            for (var i=0; i<script.length; i++)
            {
                if (script[i].parentNode == this.xml) // must be child of <plugin>
                {
                    path = this.path+script[i].getAttribute('src');
                    try {
                        text = fileio.getFileContents(path);
                    
                        if (text)
                        {
                            this.script += text + ';';
                        }
                    }
                    catch(e)
                    {
                        opera.postError('Could not open: '+path+'\n'+e)
                    }
                }
            }
            
            callback(this);
        },
        
        /**
         * Checks the variables that the plugin inspects, if the
         * value is different to the one shown in the plugin's ui
         * then the ui is updated.
         * Call regularly for best result.
         */
        
        poll: function()
        {
            for (var i=0; i<this.vars.length; i++)
            {
                showVarValue(this.vars[i]);
            }
        },
        
        /**
         * Starts a plugin running in a given widget window.
         * Performs the checking of the plugin.xml file and construction of the
         * plugin's UI elements.
         *
         * @param win {Object} The window object of the widget.
         * @returns {DocumentFragment} The plugin's UI or null if none exists.
         */
        
        start: function(win)
        {
            // peacekeeping
            if (this.xml.tagName.toLowerCase() != 'plugin') { throw 'Must have <plugin> tag'; }
            
            var pluginName = this.xml.getAttribute('name');
            var str = '';
            
            var scope = scopes[pluginName] = {
                styles: [],
                eval: null,
                arg: null,
                plugin: this
            };
            
            /*
             Import the <style>
             */
            var style = this.xml.getElementsByTagName('style');
            var ele = null;
            for (var i=0; i<style.length; i++)
            {
                if (style[i].parentNode == this.xml) // must be child of <plugin>
                {
                    ele = document.createElement('style');
                    ele.innerHTML = '@import "'+this.path+style[i].getAttribute('src')+'";';
                    document.getElementsByTagName('head')[0].appendChild(ele);
                    scope.styles.push(ele);
                }
            }
            
            var funct = new win.Function('plugin',this.script + ';return function(){return eval(arguments[0])};');
            var arg = new PluginArgument(this);          
            
            scope.eval = funct(arg);
            scope.arg = arg;
            
            /*
              Explaining the logic above: Using Function() instead of eval()
              allows the variables in the script to be local, rather than global.
              
              The return value of the function, which is wrapping eval() is essential,
              as it is the only way to get acess to that scope, to be able to read
              and write those local variables.
            */
            
            var html = document.createDocumentFragment();
            var ui = this.xml.getElementsByTagName('ui')[0];
            if (ui)
            {
                var elements = ui.getElementsByTagName('*');
                var count = 0;
                
                for (var i=0; i<elements.length; i++)
                {
                    str = elements[i].getAttribute('jsplugin');
                    
                    if (str && !Control.device.widget.JSPlugins[str])
                    {
                        continue; // jsplugin required for this variable
                    }
                    
                    switch (elements[i].tagName)
                    {
                        case 'var' :
                            count++;
                            
                            html.appendChild(variableHandler.create(elements[i], pluginName, this));
                            break;
                        
                        case 'function' :
                            count++;
                            html.appendChild(createFunction(elements[i], pluginName));
                            break;
                    }
                }
                
            }
            
            scope.arg._dispatchEvent('load', null);
            if (scope.arg.onload)
            {
                scope.arg.onload();
            }
            
            return count ? html : null;
        }
    }
    
    /**
     * PluginArgument
     * 
     * Creates a blank 'plugin' object to be passed to
     * the plugin as an argument.
     *
     * Has special methods and constants for plugin use.
     *
     * @constructor
     */
    
    function PluginArgument(plugin)
    {
        this.window = window;
        this.onunload = null;
        this.configXML = Control.device.widget.configXML;
        this.JSPlugins = Control.device.widget.JSPlugins;
        this.path = plugin.path;
        this.device = Control.device.settings;
        
        this.chrome = [
            g('screen-chrome-top'),
            g('screen-chrome-right'),
            g('screen-chrome-bottom'),
            g('screen-chrome-left')
        ];
        
        /* internal */
        this._eventListeners = {};
    }
    
    PluginArgument.prototype = {
        
        /**
         * Creates a question for the user
         *
         * @param title {String} The title of the question
         * @param text {String} The body message
         * @param checkbox {String} If set, then a label to apply to a checkbox question
         * @param callback {String} The callback, gets called with 'true' or 'false'
         */
        
        confirm: function(title, text, checkbox, callback)
        {
            title = (title || 'Prompt') + '';
            text = (text || '') + '';
            checkbox = (checkbox || '') + '';
            ui.confirm(title, text, checkbox, callback);
        },
        
        /**
         * No longer needed, supported for legacy.
         * Does the exact same as confirm()
         * DEPRECATED.
         */
        
        prompt: function()
        {
            this.confirm.apply(this, arguments);
        },

        /**
         * Creates a question for the user
         *
         * @param title {String} The title of the alert
         * @param text {String} The body message
         * @param callback {String} Optional callback
         */
        
        alert: function(title, text, callback)
        {
            title = (title || 'Alert') + '';
            text = (text || '') + '';
            ui.alert(title, text, callback);
        },
        
        /**
         * Causes the widget to go into docked mode if it is not already
         * @returns {Boolean} Whether or not the widget is docked after the operation
         */
        
        dock: function()
        {
            if (Control.device.widget.mode !='docked')
            {
                emulator.toggleDocked();
            }
            return (Control.device.widget.mode =='docked');
        },
        
        /**
         * Undocks the widget if it is in docked mode
         */
        
        undock: function()
        {
            if (Control.device.widget.mode =='docked')
            {
                emulator.toggleDocked();
            }
        },

        /**
         * Chooses the default widget-mode for the widget.
         * Normally it is 'widget', but for some devices
         * it will be 'application'.
         *
         * @param mode {String} The name of the new default mode
         */
        
        setDefaultWidgetMode: function(mode)
        {
            if (!mode || !(mode += ''))
            {
                throw new Error('Invalid "mode" attribute.');
            }
            emulator.setDefaultWidgetMode(mode);
        },
        
        /**
         * Sets the device's orientation (assuming the device allows orientation changes)
         * Throws an error if 'orientation' string parameter is not valid.
         * Posts a message to the console if portrait or landscape is requested but the device cannot rotate
         *
         * @param orientation {String} 'portrait', 'landscape' or 'auto' (empty string acts as auto)
         * @param mode {Boolean} Disables the rotate screen button if true
         */
        
        setScreenOrientation: function(orientation, disableRotateButton)
        {
            emulator.setScreenOrientation(orientation, disableRotateButton);   
        },        
        
        /** following code taken from Runeh's eventMixin.js */
        
        /**
        * Add an event listener for a given event.
        * @argument name {String} The name of the event to register
        * @argument callback {Function} The function to call when event is raised
        */
        addEventListener: function(name, callback) {
            if (name in this._eventListeners) {
                this._eventListeners[name].push(callback);
            } else {
                this._eventListeners[name] = [callback]
            }
        },
        /**
         * Remove a listener for a given event.
         * @argument name {String} The name of the event to register
         * @argument callback {Function} The function to call when event is raised
         */
        removeEventListener: function(name, callback) {
            if (!(name in this._eventListeners)) {return};
            var listeners = this._eventListeners[name];
            for (var n=listeners.length-1, e; e=listeners[n]; n--) {
                if (e==callback) { listeners.splice(n, 1); }
            }
        },
        /**
         * Dispatch an event
         * Internal
         */
        _dispatchEvent: function(name, payload) {
            if (!(name in this._eventListeners)) { return }
            var handlers = this._eventListeners[name];
            for (var n=0, e; e=handlers[n]; n++) {
                e(payload);
            }
        }
    };
    
    var scopes = {}; // key is the plugin name, value is a function
    
    /**
     * Escapes a variable name
     *
     * <p>This will transfor a variable name described as a string
     * to an identifier literal, so for example:</p>
     *
     * <ul>
     *  <li>"foo" -> "\u0066\u006f\u006f"
     *  <li>"foo bar" -> "\u0066\u006f\u006f\u0020\u0062\u0061\u0072"
     * </ul>
     *
     * <p>This is because scope functions use eval() to set and retrieve
     * variables, and if there is a nasty character in there, such
     * as a quote, then an error would have been thrown.</p>
     *
     * <p>It will not escape the '.' character though, since that is
     * used to refer to properties.</p>
     *
     * @param str {String} The variable name to escape
     * @returns {String} The escaped variable name
     */
    
    var escapeVar = (function()
    {
        var regexp = /[^.]/g;
        function funct(str)
        {
            str = str.charCodeAt(0).toString(16);
            while (str.length<4) str = '0'+str;
            return '\\u'+str;
        }
        return function(varName)
        {
            return varName.replace(regexp, funct);
        }
    })();
    
    /**
     * Creates a plugin's UI input element for the panel.
     * @param ele {Element} The xml element declaring a variable to watch
     * @param defaultValue {String} The variable's default value.
     */
    
    function createInput(ele, defaultValue)
    {
        // type is case insensitive, default to string
        var type = (ele.getAttribute('type')||'string').toLowerCase()
        var input = null;
        
        // disposable vars:
        var i=0;
        var options = null;
        var tempEle = null;
        
        switch (type)
        {
            case 'string' :
                input = document.createElement('input');
                input.setAttribute('type','text');
                input.setAttribute('value', defaultValue || '');
                break;
            
            case 'output' :
                input = document.createElement('output');
                input.setAttribute('value', defaultValue+'');
                break;
            
            case 'number' :
                input = document.createElement('input');
                input.setAttribute('type','number');
                input.setAttribute('value', defaultValue ? (Number(defaultValue)||0).toString() : '0')
                break;
            
            case 'boolean' :
                input = document.createElement('input');
                input.setAttribute('type', 'checkbox');
                if (defaultValue=='true')
                {
                    input.setAttribute('checked','checked');
                }
                break;
            
            case 'date' :
                input = document.createElement('input');
                input.setAttribute('type','date');
                break;
            
            case 'select' :
                input = document.createElement('select');
                options = ele.getElementsByTagName('option');
                for (i=0; i<options.length; i++)
                {
                    tempEle = document.createElement('option');
                    tempEle.setAttribute('_varType', (options[i].getAttribute('type')||'string').toLowerCase());
                    tempEle.setAttribute('value',options[i].getAttribute('value'));
                    tempEle.appendChild(
                        document.createTextNode(
                            options[i].getAttribute('title') ||
                            options[i].getAttribute('value')));
                    if (options[i].getAttribute('selected') == 'selected')
                    {
                        tempEle.setAttribute('selected','selected');
                    }
                    input.appendChild(tempEle);
                }
                break;
            
            default :
                throw new Error('Unknown variable type in plugin');
        }
        
        input.setAttribute('_varType', type);
        
        return input;
    }
    
    /**
     * Will update an element's value with the one probed from the
     * plugin. This will not update the value if the element is
     * focued (unless force is true).
     *
     * @param input {Element} The input element that needs updating
     * @param force {Boolean} If true then the condition that the element be blurred is relaxed
     }
     */
    
    function showVarValue(input, force)
    {
        var value = null;
        var i=0;
        
        if (!force && input.hasAttribute('data-focused'))
        {
            return; // don't update focued elements
        }
        
        try {
            value = scopes[input.getAttribute('_plugin')].eval(input.getAttribute('_varName'));
        }
        catch(e)
        {
            value = null;
        }
        
        switch (input.getAttribute('_varType'))
        {
            case 'boolean' :
                value = !!value;
                if (input.checked != value)
                {
                    input.checked = value;
                    variableHandler.varChanged(input);
                }
                break;
            default :
                if (input.value !== value)
                {
                    input.value = value;
                    variableHandler.varChanged(input);
                }
                break;
        }
    }
    
    /**
     * Extracts the value of an plugin's UI input
     *
     * @param ele {Element} The element to get the value from
     * @returns {Object} The string/number/boolean whatever that the input has as a value
     */
    
    function resolveInputValue(ele)
    {
        var value = null;
        
        switch (ele.getAttribute('_varType'))
        {
            case 'string' :
                value = ele.value;
                break;
            case 'number' :
                value = Number(ele.value);
                ele.value = value || 0;
                break;
            case 'date' :
                // todo
                break;
            case 'boolean' :
                value = ele.checked;
                break;
            case 'select' :
                ele = ele.options[ele.options.selectedIndex];
                switch (ele.getAttribute('_varType'))
                {
                    case 'string' :
                        value = ele.value;
                        break;
                    case 'number' :
                        value = Number(ele.value);
                        break;
                    case 'date' :
                    case 'boolean' :
                    case 'select' :
                        throw new Error('<option> element cannot have types: "string", "boolean", "select".'); // this should never throw
                        break;
                    default :
                        throw new Error('Unknown varType: '+ele.getAttribute('_varType')); // this should never throw
                }  
                break;
            default :
                throw new Error('Unknown varType: '+ele.getAttribute('_varType')); // this should never throw
        }
        return value;
    }
    
    /**
     * Turns a JS value into its source representation.
     * Eg, a string containing foo will return "foo"
     * A number will just return its value etc.
     *
     * @returns {String} The variable described as a string.
     */
    
    function stringifyValue(value)
    {
        switch (typeof value)
        {
            case 'string' :
                value = '"'+value.replace('\\','\\\\').replace(/\"/,'\\"')+'"';
                break;
            case 'number' :
                value = Number(value).toString();
                break;
            case 'date' :
                // todo
                break;
            case 'boolean' :
                value = value + '';
                break;
            default :
                throw new Error('Unknown varType'); // this should never throw
        }
        return value;
    }
    
    var variableHandler = new function()
    {
        var self = this;
        var savedValues = widget.preferenceForKey('pluginValues') || {};
        
        if (typeof savedValues == 'string')
        {
            savedValues = {};
        }
        function focusEvent(e)
        {
            this.setAttribute('_oldValue',this.value);
            this.setAttribute('data-focused', '1');
        }
        function blurEvent(e)
        {
            this.removeAttribute('data-focused');
            showVarValue(this);
        }
        /**
         * This function handles enter and key presses (perhaps in future others).
         * To make the the UI elements easier to use.
         */
        function keyEvent(e)
        {
            var ele = this;
            
            var varType = this.getAttribute('_varType');
            
            if (varType == 'string' || varType == 'number')
            {
                switch (e.keyCode)
                {
                    case 13 : // enter-key, set the value
                        changeEvent.call(this);
                        showVarValue(ele, true);
                        this.select();
                        break;
                    case 27 : // escape, return to previous value
                        this.value = this.getAttribute('_oldValue');
                        this.select();
                        break;
                }
            }
        }
        function changeEvent(e)
        {
            var varName = this.getAttribute('_varName');
            var varType = this.getAttribute('_varType');
            var pluginName = this.getAttribute('_plugin');
            var onChange = this.getAttribute('_onchange');
            var valueString = stringifyValue(resolveInputValue(this));
            
            try {
                scopes[pluginName].eval(escapeVar(varName)+'='+valueString);
            }
            catch(e)
            {
                throw new Error('variable could not be set');
                return;
            }
            
            self.varChanged(this);
            
            if (onChange)
            {
                try {
                    scopes[pluginName].eval(escapeVar(onChange)+'('+valueString+')');
                }
                catch(e)
                {
                    throw new Error('Error thrown in variable ('+varName+') onchange event handler:\n'+e);
                }
            }
        }
        this.varChanged = function(ele)
        {
            var varName = ele.getAttribute('_varName');
            var pluginName = ele.getAttribute('_plugin');
            var value = null;
            
            try {
                value = scopes[pluginName].eval(varName);
            }
            catch(e) {
                return;
            }
            
            if (!savedValues[pluginName])
            {
                savedValues[pluginName] = {};
            }
            // store the new value as a preference:
            if (savedValues[pluginName][varName] != value)
            {
                savedValues[pluginName][varName] = value;
                widget.setPreferenceForKey(savedValues, 'pluginValues');
            }
        }
        function loadSavedValue(input, pluginName, varName)
        {
            var value = null;
            if ( savedValues[pluginName] && savedValues[pluginName].hasOwnProperty(varName))
            {
                
                try {
                    value = savedValues[pluginName][varName];
                    
                    scopes[input.getAttribute('_plugin')].eval(escapeVar(varName)+'='+stringifyValue(value));
                }
                catch (e)
                {
                    opera.postError(e);
                    return false;
                }
                
                switch (input.getAttribute('_varType'))
                {
                    case 'boolean' :
                        input.checked = !!value;
                        break;
                    case 'number' :
                        input.value = Number(value)||0;
                        break;
                    default :
                        input.value = value;
                        break;
                    
                }
                
                return true;
            }
            return false;
        }
        
        this.create = function(ele, pluginName, plugin)
        {
            var label = document.createElement('label');
            var span = document.createElement('span');
            
            var input = createInput(ele);
            var varName = ele.getAttribute('name');
            // set personal attributes so that they can
            // share the same event handlers
            input.setAttribute('_varName', varName);
            
            input.setAttribute('_plugin', pluginName);
            
            if (ele.getAttribute('onchange'))
            {
                input.setAttribute('_onchange', ele.getAttribute('onchange'));
            }
            
            input.onfocus = focusEvent;
            input.onblur = blurEvent;
            input.onkeypress = keyEvent;
            input.onchange = changeEvent;
            
            plugin.vars.push(input);
            
            if (ele.getAttribute('remember') !='true' ||
                !loadSavedValue(input, pluginName, varName))
            {
                showVarValue(input); // show it's current value
            }
            
            span.appendChild(document.createTextNode(ele.getAttribute('title')));
            label.appendChild(span);
            label.appendChild(input);
            
            return label;
        };
    };
    
    /**
     * Creates form for a function
     *
     * Constructs all elements and
     * event handlers.
     *
     * @param ele {Element} The SVG element describing the function
     * @returns The form element
     */
    
    var createFunction = (function()
    {
        function submitEvent(e)
        {
            var functionName = this.getAttribute('_functionName');
            
            var args = this.getElementsByClassName('arg');
            var arr = [];
            
            for (var i=0; i<args.length; i++)
            {
                arr[i] = stringifyValue(resolveInputValue(args[i]));
            }
            
            try
            {
                scopes[this.getAttribute('_plugin')].eval(escapeVar(functionName)+'('+arr.join(',')+')');
            }
            catch (err)
            {
                opera.postError(err);
                throw new Error('Failed to call the function: ');
                
            }
            
            e.preventDefault();
            return false;
        }
        
        function createArg(arg)
        {
            var label = document.createElement('label');
            var span = document.createElement('span');
            
            var input = createInput(arg, arg.getAttribute('value'));
            
            input.setAttribute('class', 'arg'); // so we can search for them
            
            span.appendChild(document.createTextNode(arg.getAttribute('title')));
            
            label.appendChild(span);
            label.appendChild(input);
            
            return label;
        }
        
        /*
         The function below is the function 'createFunction'
         (Currying technique)
        */
        
        return function(ele, pluginName)
        {
            var form = document.createElement('form');
            var fieldset = document.createElement('fieldset');
            var legend = document.createElement('legend');
            var submit = document.createElement('input');
            var input = null;
            
            var args = ele.getElementsByTagName('arg');
            var i=0;
            
            legend.appendChild(document.createTextNode(ele.getAttribute('title')));
            fieldset.appendChild(legend);
            
            for (i=0; i<args.length; i++)
            {
                fieldset.appendChild(createArg(args[i]));
            }
            
            submit.type = 'submit';
            submit.value = ele.getAttribute('button');
            
            form.setAttribute('_plugin', pluginName);
            form.setAttribute('_functionName', ele.getAttribute('name'));
            
            fieldset.appendChild(submit);
            form.appendChild(fieldset);
            
            form.onsubmit = submitEvent;
            
            return form;
        }
    })();
    
    function resolveObject(win, namespace)
    {
        if (!namespace) { return {} }
        
        var arr = namespace.split('.');
        for (var i=0; i<arr.length; i++)
        {
            if (!arr[i]) { throw 'Invalid namespace for <object>'}
            if (!win[arr[i]])
            {
                win[arr[i]] = {};
            }
            win = win[arr[i]];
        }
        return win;
    }
    
    var poller = (function()
    {
        var interval = 0;
        
        var self = {
            start: function()
            {
                if (interval)
                {
                    return; // already started, no need to start again
                }
                interval = setInterval(loop, 500);
            },
            stop: function()
            {
                if (!interval)
                {
                    return; // already stopped, no need to stop again
                }
                clearInterval(interval);
                interval = 0;
            }
        }
        
        function loop()
        {
            for (var name in pluginHandler.plugins)
            {
                pluginHandler.plugins[name].poll();
            }
        }
        
        return self;
    })();
    
    return {
        
        plugins: {},
        
        /**
         * @param callback {Function} called once all plugins are loaded
         */
        
        init: function(callback)
        {
            var list = fileio.getPluginsList();
            
            if (list.length == 0)
            {
                callback();
                return;
            }
            
            var loadedPlugins = 0;
            
            for (var i=0; i<list.length; i++)
            {
                pluginHandler.preparePlugin(list[i], onload);
            }
            
            function onload()
            {
                loadedPlugins++
                if (loadedPlugins == list.length)
                {
                    callback();
                }
            }
        },
        
        /**
         * Starts the process of loading a plugin.
         * Reads its name, properties, scripts etc.
         * Also adds it to the plugins hash table.
         *
         * @param path {String} The 'mountpoint://...' path of the plugin
         * @param callback {Function} Called once all preparing is done, passes the new Plugin object as the first argument
         */
        
        preparePlugin: function(path, callback)
        {
            new Plugin(path, callback);
        },
        
        /**
         * Deletes a plugin from the lineup.
         * If the plugin is actively running in a widget it may continue to
         * still run until the widget is reloaded.
         */
        
        deletePlugin: function(name)
        {
            var plugin = this.plugins[name];
            
            if (plugin)
            {
                delete this.plugins[name];
            }
        },
        
        /**
         * Starts a plugin running on a widget
         *
         * @param name {String} The name of the plugin to run.
         * @param frameWindow {Object} The window object of the widget
         * @returns {Object} DocumentFragment of the plugin's UI
         * @throws Error 'Plugin not found' if the name doesn't match any prepared plugins
         */
        
        startPlugin: function(name, frameWindow)
        {
            var plugin = null;
            
            if (this.plugins[name])
            {
                if (this.plugins[name].group)
                {
                    /* group specified, only run if no other plugin of that
                      group is running */
                    for (var j in scopes)
                    {
                        if (scopes[j].plugin.group == this.plugins[name].group)
                        {
                            throw new Error('Plugin ('+name+') not loaded due to group conflict: '+this.plugins[name].group+'.');
                        }
                    }
                }
                plugin = this.plugins[name].start(frameWindow);
                
                poller.start();
                return plugin;
            }
            
            throw new Error('Plugin not found: '+name);
            return null;
        },
        
        /**
         * Stops all plugins running on the current widget
         * Calls necessary unload events.
         * Removes relevant plugin chrome.
         */
        
        closeAllPlugins: function()
        {
          var scope;
          var i;
          var j;
          
          for (i=0; i<Control.device.plugins.length; i++)
          {
            scope = scopes[Control.device.plugins[i]];
            if (!scope) { continue; }
            
            // dispatch unload event
            try
            {
                scope.arg._dispatchEvent('unload', null);
                if (scope.arg.onunload)
                {
                    scope.arg.onunload();
                }
            }
            catch(err)
            {
                opera.postError(err);
            }
            
            // remove the styles that were added
            for (j=0; j<scope.styles.length; j++)
            {
                scope.styles[j].parentNode.removeChild(scope.styles[j]);
            }
            
            delete scopes[Control.device.plugins[i]];
          }
          
          // reset the chrome elements
          // the only way to be sure they are completely cleared of styles+properties
          // is to remove them and add new divs
          
          var chromeNames = ['top','right','bottom','left'];
          var parent = g('screen-chrome');
          var ele = null;
          for (var i=0; i<4; i++)
          {
            parent.removeChild(g('screen-chrome-'+chromeNames[i]));
            ele = document.createElement('div');
            ele.setAttribute('id','screen-chrome-'+chromeNames[i]);
            parent.appendChild(ele);
            
          }
          
          g('plugin-ui').innerHTML = '';
          
          poller.stop();
        }
    }
    
})();
